import os
import sys

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

from ciceroscm import concentrations_emissions_handler, input_handler

# Change data_dir to get inputs from somewhere else
data_dir = os.path.join(os.path.dirname(__file__), "..", "..", "tests", "test-data")

# Change these years if you wish to adjust the estimated time range:
nyend = 2500
nystart = 1750

# Choose the lifetime mode assumption for methane 
# lifetime_modes = ["TAR", "CONSTANT_12", "CONSTANT_from_file", "WIGLEY"]
# Change this line to your preferred lifetime:
lf_mode = "TAR"

pamset = {"nyend": nyend, "nystart": nystart, "lifetime_mode": lf_mode}
ih_temp = input_handler.InputHandler(pamset)

# If you want to fit from different data input files, then change these paths
em_data = ih_temp.read_emissions(os.path.join(data_dir, "ssp245_em_RCMIP.txt"))
conc_data =  input_handler.read_inputfile(os.path.join(data_dir, "ssp245_conc_RCMIP.txt"))
gaspam_data = input_handler.read_components(os.path.join(data_dir, "gases_vupdate_2022_AR6.txt"))

def get_lifetime(tracer, yr, ce_handler):
    """
    Get lifetime for tracer as calculated in the code
    """
    q = 1/ce_handler.df_gas["TAU1"][tracer]
    if tracer != "CH4":
        return q
    q = ce_handler.methane_lifetime(q, ce_handler.conc_in[tracer][yr-1], yr)
    return q

def old_get_nat_em_timeseries(tau, beta, em_nat, em_series, conc_series, sp="N2O"):
    """
    Get natural emissions using the previously used old natural emissions
    method. 
    
    We recommend using the ode method instead for more efficient
    and accurate calculations
    """
    em_nat_hist = np.full(len(conc_series), em_nat,np.double)
    CONC_NEW_last = conc_series[0]
    for i in range(len(conc_series)):
        not_done = True
        times_around = 0
        print(i)
        q = get_q(sp, i+nystart)
        
        while not_done: #and times_around < 100:
            pc = (em_series[i] + em_nat)/(beta)
            CONC_NEW = (pc/q+(CONC_NEW_last-pc/q)*np.exp(-q*1.0))
            print(CONC_NEW)
            print(conc_series[i])
            if (CONC_NEW/conc_series[i] >  1.005): 
                em_nat = em_nat*0.995
            elif (CONC_NEW/conc_series[i] < 0.995): 
                em_nat = em_nat*1.005
            else:
                em_nat_hist[i] = em_nat
                not_done = False
            times_around = times_around +1
        CONC_NEW_last = CONC_NEW
    return em_nat_hist

def ode_get_nat_em_timeseries(tau, beta, em_series, conc_series, ce_handler, sp = "N2O"):
    """
    Use ode exact solution backwards to get natural emissions
    needed to match the concentrations from anthropogenic emissions
    series
    """
    conc_extended = np.concatenate(([conc_series[0]], conc_series), axis = 0)
    if sp != "CH4":
        em_nat_hist = beta/tau/(1-np.exp(-1/tau))*(conc_series -conc_extended[:-1]*np.exp(-1/tau)) - em_series[:len(conc_series)]
    else:
        em_nat_hist = np.zeros(len(conc_series))
        for i in range(len(conc_series)):
            q = get_lifetime(sp, i+nystart, ce_handler)
            em_nat_hist[i] = beta*q/(1-np.exp(-q))*(conc_series[i] - conc_extended[i]*np.exp(-q)) - em_series[i]
    return em_nat_hist



nat_ch4_data = pd.DataFrame(
                data={"CH4": np.ones(nyend - nystart + 1) * 242.09},
                index=np.arange(nystart, nyend + 1),
            )
nat_n2o_data = pd.DataFrame(
                data={"CH4": np.ones(nyend - nystart + 1) * 242.09},
                index=np.arange(nystart, nyend + 1),
            )
ce_handler = concentrations_emissions_handler.ConcentrationsEmissionsHandler(input_handler.InputHandler({"gaspam_data":gaspam_data, "emissions_data":em_data, "concentrations_data": conc_data, "nat_ch4_data": nat_ch4_data, "nat_n2o_data": nat_n2o_data, "nystart": nystart, "nyend": nyend}), pamset)

ce_handler.reset_with_new_pams(ce_handler.pamset)
conc_y0 = ce_handler.conc_in.index.values[0]

# In principle you could use this for other tracers as well or do just a single tracer at a time
tracers = ["N2O", "CH4"]
year_out = range(nystart, nyend + 1)
for j, tracer in enumerate(tracers):
    tau = ce_handler.df_gas["TAU1"][tracer]
    beta = ce_handler.df_gas["BETA"][tracer]
    em_nat_const = ce_handler.df_gas["NAT_EM"][tracer]
    em_series = ce_handler.emis[tracer].values
    conc_series = ce_handler.conc_in[tracer][nystart-conc_y0:nyend+1-conc_y0].values
    print(print(conc_series))
    print(ce_handler.conc_in[tracer])

    # Option for old version calculation (comment in next three lines for old version):
    # em_nat_hist_old = old_get_nat_em_timeseries(tau, beta, em_nat_const, em_series, conc_series, sp=tracer)
    # em_nat_out_old = np.concatenate((em_nat_hist_old,  np.full((len(year_out)-len(em_nat_hist_old)), np.mean(em_nat_hist_old[-11:]),np.double)), axis=0)
    # np.savetxt(f"natemis_{tracer}_old_method_from_rcmip.txt", em_nat_out_old,fmt='%1.4f')

    # Option for new (recommended) version calculation (comment out next three lines for old version):
    em_nat_hist_ode = ode_get_nat_em_timeseries(tau, beta, em_series, conc_series, ce_handler, sp=tracer)
    em_nat_out_ode = np.concatenate((em_nat_hist_ode,  np.full((len(year_out)-len(em_nat_hist_ode)), np.mean(em_nat_hist_ode[-11:]),np.double)), axis=0)
    np.savetxt(f"natemis_{tracer}_ode_method_from_rcmip.txt", em_nat_out_ode,fmt='%1.4f')





"""
Module to perform calibration of the ciceroscm model
"""

import logging

import numpy as np
import pandas as pd

from .distributionrun import DistributionRun

LOGGER = logging.getLogger(__name__)
options_default = {"subsamplesize": 50, "maxrecurse": 4}


class Calibrator:
    """
    Calibrator class used to do calibration
    """

    def __init__(
        self, calibdata, config_distro, scendata, options=options_default
    ):  # pylint:disable=dangerous-default-value
        """
        Initialise calibrator
        Set the calibrators data to calibrate to, prior, scenario
        and subsample size

        Parameters
        ----------
        calibdata : pandas.DataFrame
            Data Frame containing datapoints to calibrate to
            For each datapoint the following columns should be filled
            in: 'Variable Name', which should be the name of the variable
            to be fitted, this should be an ScmRun variable. 'Yearstart_norm'
            'Yearend_norm', 'Yearstart_change' and 'Yearend_change'.
            These assume that the variable has an expected change between
            a norm period and a change period, and define the inclusive
            ranges of these extents. If the variable change is just between
            single years, set Yearstart_norm and Yearend_norm equal.
            'Central value' is the observed change in the variable and
            finally 'sigma' is the standard deviation in the observed
            value
        config_distro: :obj: ConfigDistro
            An object of the ConfigDistro object which has defined priors
            an set values, and can produce draws for the calibrator
            to pick from
        scendata: dict
            Dictionary that defines what to run in order to calibrate
        options: dict
            Dictionary including keyword options that the user can
            choose to set. These are subsamplesize and maxrecurse
            The calibrator will try to get fitted values by
            sampling in chunks and doing an accept/reject on the samples
            after running them. This will be done recursively untill the
            maximal recursion depth decided by maxrecurse is reached,
            or the requested number of accepted samples is found.
            The subsamplesize decides the number of configuration in
            each chunk.
        """
        self.calibdata = calibdata
        self.subsamplesize = options["subsamplesize"]
        self.config_distro = config_distro
        self.scendata = scendata
        self.recurse_max = options["maxrecurse"]

    def assign_log_liklihood(self, res):
        """
        Assign a log likelihood for a result

        From the calibdata determine a log likelihood for
        a results run

        Parameters
        ----------
        res: ScmRun
            A filtered ScmRun for a single configuration run

        Returns
        -------
            float
            The log likelihood for the result matching the
            calibration data
        """
        logli = 0
        for (
            index,  # pylint: disable=unused-variable
            datapoint,
        ) in self.calibdata.iterrows():
            vres = (
                res.filter(
                    variable=datapoint["Variable Name"], year=datapoint["Yearend"]
                ).values[0]
                - res.filter(
                    variable=datapoint["Variable Name"], year=datapoint["Yearstart"]
                ).values[0]
            )
            sigma = datapoint["sigma"]
            vexp = datapoint["Central Value"]
            logli = logli - 0.5 * (vres - vexp) ** 2 / sigma**2  # + np.log(sigma)
        return logli

    def find_distance(self, res):
        """
        Find the probability scaled distance between result
        and calibration data

        Using the distance measure Sum (x-mu)**2/sigma**2
        where the sum is over each of the datapoints in the calibration
        data

        Parameters
        ----------
        res: ScmRun
            A filtered ScmRun for a single configuration run

        Returns
        -------
            float
            The probability distance between the result and the
            calibration data
        """
        distance = 0
        for (
            index,  # pylint: disable=unused-variable
            datapoint,
        ) in self.calibdata.iterrows():
            if datapoint["Yearstart_change"] == datapoint["Yearend_change"]:
                vchange = res.filter(
                    variable=datapoint["Variable Name"],
                    year=datapoint["Yearstart_change"],
                ).values[0]
            else:
                vchange = np.mean(
                    res.filter(
                        variable=datapoint["Variable Name"],
                        year=range(
                            datapoint["Yearstart_change"], datapoint["Yearend_change"]
                        ),
                    )
                )
            if datapoint["Yearstart_norm"] == datapoint["Yearend_norm"]:
                vnorm = res.filter(
                    variable=datapoint["Variable Name"],
                    year=datapoint["Yearstart_norm"],
                ).values[0]
            else:
                vnorm = np.mean(
                    res.filter(
                        variable=datapoint["Variable Name"],
                        year=range(
                            datapoint["Yearstart_norm"], datapoint["Yearend_norm"]
                        ),
                    ).values
                )
            vres = vchange - vnorm
            sigma = datapoint["sigma"]
            vexp = datapoint["Central Value"]
            distance = distance + (vres - vexp) ** 2 / sigma**2
        return distance

    def reject_samples(self, results):
        """
        Reject and accept samples probabilistically

        Sample distances to observations are compared
        to the distance between two randomally drawn
        samples from a normal distribution multiplied
        with the datapoint dimensionality. If the distance
        is less than the random draw, the sample is accepted
        otherwise the sample is rejected

        Parameters
        ----------
        results: ScmRun

        ScmRun for a chunk of model configurations

        Returns
        -------
            list
            A list of the configuration indices that passed
            the test
        """
        indices = results.get_unique_meta("run_id")

        draws = np.square(
            np.random.normal(size=(len(indices)))
            - np.random.normal(size=(len(indices)))
        )
        keep = []
        for i, index in enumerate(indices):
            distance = self.find_distance(results.filter(run_id=index))
            if draws[i] * len(self.calibdata) > distance:
                keep.append(index)
        return keep

    def get_n_samples(self, numvalues, current_samples, kept_configs, recurse_num):
        """
        Recurse untill numvalues samples are found

        This method creates a new distribution of the object samplesize
        and runs over it if the required number of values to the
        distribution is reached after this run, it returns,
        otherwise it calls it self and increases the recursion depth

        Parameters
        ----------
        numvalues: int
            Number of accepted values requested
        current_samples: list
            Index list of samples kept
        kept_configs: list
            List of configuration dictionaries accepted
        recurse_num: int
            Current recursion depth. Defines when to stop
            recursion to avoid infinite recursion

        Returns
        -------
            list
            list of accepted configuration dictionaries
        """
        distro_run = DistributionRun(
            self.config_distro,
            indexer_pre=f"{recurse_num}_",
            numvalues=self.subsamplesize,
        )
        if isinstance(self.calibdata, pd.DataFrame):
            print(self.calibdata.shape)
            output_vars = self.calibdata["Variable Name"].unique()
        else:
            output_vars = self.calibdata["Variable Name"]
        results = distro_run.run_over_distribution(self.scendata, output_vars)
        kept = self.reject_samples(results)
        current_samples.extend(kept)
        for item in distro_run.cfgs:
            if item["Index"] in kept:
                kept_configs.append(item)
        if len(current_samples) >= numvalues:
            return kept_configs
        if recurse_num > self.recurse_max:
            LOGGER.warning(
                "Reached recurse limit %d with only %d",
                self.recurse_max,
                len(current_samples),
            )
            return kept_configs
        return self.get_n_samples(
            numvalues, current_samples, kept_configs, recurse_num + 1
        )



"""
Perturbation related methods
"""

import numpy as np
import pandas as pd


def calculate_hemispheric_forcing(tracer, q, forc_nh, forc_sh):
    """
    Calculate hemispheric forcing per tracer

    Particular hemisperic factors ar applied for
    SO2, SO4_IND and TROP_O3
    For other compounds the split is equal

    Parameters
    ----------
    tracer : str
          Name of tracer for which to do the split
    q : float
      Global tracer forcing
    forc_nh : float
           Northern hemispheric forcing
    forc_sh : float
           Southern hemispheric forcing

    Returns
    -------
    list
        Containing the two updated hemispheric forcings
        forc_nh, forc_sh
    """
    if tracer in (
        "SO2",
        "SO4_IND",
        "BC",
        "OC",
        "BMB_AEROS",
        "BMB_AEROS_OC",
        "BMB_AEROS_BC",
    ):
        # Smith et al 2020
        forc_nh = forc_nh + q * 1.47
        forc_sh = forc_sh + q * (2 - 1.47)
    elif tracer == "TROP_O3":
        # Skeie et al 2020
        forc_nh = forc_nh + q * 1.45  # 1.3
        forc_sh = forc_sh + q * (2 - 1.45)  # 0.7
    elif tracer == "LANDUSE":
        # Smith et al 2020
        forc_nh = forc_nh + q * 1.42
        forc_sh = forc_sh + q * (2 - 1.42)
    else:
        forc_nh = forc_nh + q
        forc_sh = forc_sh + q
    return forc_nh, forc_sh


def perturb_emissions(input_handler, emissions_df):
    """
    Add emission perturbations to emissions_df

    Read in emissions perturbations file and add
    perturbations to existing emissions

    Parameters
    ----------
    input_handler : ciceroscm.InputHandler
                    InputHandler that takes care of
                    input data
    emissions_df : pandas.Dataframe
                Dataframe of emissions

    Returns
    -------
    pandas.Dataframe
                    Dataframe of emissions with perturbed
                    emissions added in
    """
    pert_df = input_handler.get_data("perturb_em")
    for row in pert_df.itertuples(index=True, name="Pandas"):
        tracer = row.component
        if row.component == "CO2" and "CO2" not in emissions_df:
            tracer = "CO2_FF"
        emissions_df[tracer][row.year] = emissions_df[tracer][row.year] + row.emission
    # might not need to return, as the pandas should change itself
    return emissions_df


class ForcingPerturbation:
    """
    Class to handle  forcing perturbations

    Attributes
    ----------
    perturb_raw : pandas.Dataframe
                  Dataframe storing the perturbations
    years : np.ndarray
            Array of years for which perturbations exist
    compounds : np.ndarray
                Array of components for which perturbations
                exist
    year0 : int
            First year of perturbations
    """

    def __init__(self, input_handler, year0):
        """
        Initialse forcing perturbation instance

        Reading in the perturbations and the years for
        which they apply, also which compounds they apply
        for

        Parameters
        ----------
        perturbation_file : str
                         Path of perturbation file
        year0 : int
             First year of perturbations
        """
        self.perturb_raw = input_handler.get_data("perturb_forc")
        self.years = np.unique(self.perturb_raw["year"].values)
        self.compounds = pd.unique(self.perturb_raw["component"].values)
        self.year0 = year0

    def check_if_year_in_pert(self, year):
        """
        Check if year has perturbations

        Method to check if there are perturbations for a
        given year

        Parameters
        ----------
        year : int
            year to check

        Returns
        -------
        bool
            Truth value of if the year has perturbations

        """
        return bool(year in self.years)

    def check_if_compound_in_pert(self, compound):
        """
        Check if compound is perturbed at some point

        Method to check if a certain compound has perturbations

        Parameters
        ----------
        compound : str
                Name of compound to check

        Returns
        -------
        bool
            Truth value of if the compound has perturbations
        """
        return bool(compound in self.compounds)

    def add_forcing_pert(
        self, totforc, forc_nh, forc_sh, forc, yr
    ):  # pylint: disable=too-many-arguments
        """
        Add forcing perturbations to precalculated forcing

        Method to add perturbations to forcing

        Parameters
        ----------
        totforc : float
               Total forcing for current year
        forc_nh : float
                Northern hemispheric forcing
        forc_sh : float
                Southern hemispheric forcing
        forc : dict
            Of forcing per compound and year
        yr : int
          Year for which to calculate

        Returns
        -------
        list
            Containing updated total and forcing dict and
            hemispheric forcings
            Like this: totforc, forc_nh, forc_sh, forc
        """
        perturb_here = self.perturb_raw[self.perturb_raw.year == yr]
        for row in perturb_here.itertuples(index=True, name="Pandas"):
            value = row.forcing
            tracer = row.component
            totforc = totforc + value
            forc_nh, forc_sh = calculate_hemispheric_forcing(
                tracer, value, forc_nh, forc_sh
            )
            forc[tracer][yr - self.year0] = forc[tracer][yr - self.year0] + value
        return totforc, forc_nh, forc_sh, forc



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

pertfilename = 'pertforc_test.txt'

#Read components, to get the units:
inputdir = '/div/no-backup/users/ragnhibs/ciceroscm/tests/test-data/'
df_comp =pd.read_csv(inputdir + 'gases_v1RCMIP.txt', sep='\t', index_col=0)

print(df_comp)

#Choose component to perturbe:
comp = 'OTHER'
unit_test = df_comp.loc[comp]['EM_UNIT']
#Should add a check if components are not included.
#But anyway it will crash here. We do not need the unit

unit ='Wm-2'

#Make sure to make the perturbation in the correct unit.

#Can read these from file
pert_years = [2000,2001,2002]
#The forcing will be multiplied by -1 to be subtracted from the file
forcing = np.array([4,4,4])


columns = ['component','unit','source','year','forcing']

df = pd.DataFrame(columns=columns)
df['year']=pert_years
df['forcing']=forcing*-1.0
df['source'] = 'src'
df['component'] = comp
df['unit'] = unit
df = df.set_index('component')

print(df)


df.to_csv(pertfilename)





import os
import shutil

import numpy as np
import pandas as pd
import pandas.testing as pdt

from ciceroscm import CICEROSCM, input_handler


def check_output(
    output_dir, expected_output_dir, update_expected_files=False, rtol=1e-2
):
    files = ["output_temp.txt", "output_ohc.txt", "output_sunvolc.txt"]

    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)

        else:
            res = pd.read_csv(file_to_check, delim_whitespace=True)
            exp = pd.read_csv(file_expected, delim_whitespace=True)
            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def check_output_subset(
    output_dir, expected_output_dir, update_expected_files=False, rtol=1e-2
):
    files = {"output_temp.txt": [0, 1, 2, 3], "output_ohc.txt": [0, 1, 2]}

    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)

        else:
            res = pd.read_csv(
                file_to_check, delim_whitespace=True, usecols=files[filename]
            )
            exp = pd.read_csv(file_expected, delim_whitespace=True)
            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def test_ciceroscm_run(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            "gaspamfile": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "forc_data": np.loadtxt(os.path.join(test_data_dir, "test_forcing.txt")),
        },
    )
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
            "ldtime": 12,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "1_year_blipp"))

    # 1pct CO2 without sunvolc
    cscm = CICEROSCM(
        {
            "nyend": 2100,
            "forc_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
        },
    )

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "1pct_CO2_no_sunvolc"))

    ih = input_handler.InputHandler({"nyend": 2100, "sunvolc": 1})
    # 1 ppct CO2 with sunvolc
    cscm = CICEROSCM(
        {
            "nyend": 2100,
            "sunvolc": 1,
            "forc_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
            "rf_volc_n_data": ih.get_data("rf_volc_n") + 0.371457071,
            "rf_volc_s_data": ih.get_data("rf_volc_s") + 0.353195076,
        },
    )
    outdir = str(tmpdir)
    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "1pct_CO2"))
    # check_output(outdir, os.path.join(test_data_dir,"1pct_CO2_no_sunvolc"))
    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "threstemp": 0,
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output_subset(outdir, os.path.join(test_data_dir, "nr_test_1pct_CO2"))
    # Test hemispheric split:
    cscm = CICEROSCM(
        {
            "nyend": 2100,
            "forc_file": os.path.join(test_data_dir, "test_forcing_hemisplit.txt"),
        },
    )
    outdir = str(tmpdir)
    # outdir = os.path.join(os.getcwd(), "output")
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "threstemp": 0,
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "1_year_blipp"))
    # Test component split:
    cscm = CICEROSCM(
        {
            "gaspamfile": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "forc_file": os.path.join(test_data_dir, "test_forcing_components.txt"),
        },
    )
    outdir = str(tmpdir)
    # outdir = os.path.join(os.getcwd(), "output")
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "threstemp": 0,
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "1_year_blipp"))


"""
def test_cfg(test_data_dir):
    cscm = CICEROSCM()
    # cscm._run(
    #    {"gaspamfile": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),},
    #    {"forc_file": os.path.join(test_data_dir, "CO2_1pros.txt")},
    # )


#    cscm._run(
#        {"gaspamfile": os.path.join(test_data_dir, "gases_v1RCMIP.txt"), "sunvolc": 1},
#        {"forc_file": os.path.join(test_data_dir, "CO2_1pros.txt")},
#    )
"""



import os

import numpy as np

from ciceroscm import input_handler
from ciceroscm.parallel._configdistro import _ConfigDistro
from ciceroscm.parallel.distributionrun import DistributionRun


def test_distro_run_forcing(test_data_dir):
    testconfig = _ConfigDistro(
        setvalues={"threstemp": 7.0, "lm": 40, "ldtime": 12}, options={"forc": True}
    )
    jsonfile = os.path.join(test_data_dir, "test_config_json.json")
    distrorun1 = DistributionRun(testconfig, numvalues=40)
    gaspam_data = input_handler.read_components(
        os.path.join(test_data_dir, "gases_v1RCMIP.txt")
    )
    scenarios = {
        "gaspamfile": gaspam_data,
        "nyend": 2100,
        "forc_data": np.loadtxt(os.path.join(test_data_dir, "test_forcing.txt")),
        "udir": test_data_dir,
        "scenname": "forc_data_test_forcing",
    }
    output_vars = ["Heat Content|Ocean", "Surface Air Temperature Change"]
    distrorun1.write_configs_to_json(json_file_name=jsonfile)
    results1 = distrorun1.run_over_distribution(scenarios, output_vars)
    distrorun2 = DistributionRun(None, jsonfile)
    results2 = distrorun2.run_over_distribution(scenarios, output_vars)
    assert np.array_equal(results1.values, results2.values)
    os.remove(jsonfile)



import os
import sys

# Adding location of source code to system path
# os.path.dirname(__file__) gives the directory of
# current file. Put in updated path if running script from elsewhere
# os.path joins all the folders of a path together in a
# system independent way (i.e. will work equally well on Windows, linux etc)
sys.path.append(os.path.join(os.path.dirname(__file__), "../", "src"))

from ciceroscm import CICEROSCM

data_dir = os.path.join(os.path.dirname(__file__), "../", "tests", "test-data")
#os.getcwd() gets the path of where you are running from
outdir = os.path.join(os.getcwd(), "output_test")
cscm = CICEROSCM(
    {
        "gaspam_file": os.path.join(data_dir, "gases_v1RCMIP.txt"),
        "nyend": 2100,
        "concentrations_file": os.path.join(data_dir, "ssp245_conc_RCMIP.txt"),
        "emissions_file": os.path.join(data_dir, "ssp245_em_RCMIP.txt"),
        "nat_ch4_file": os.path.join(data_dir, "natemis_ch4.txt"),
        "nat_n2o_file": os.path.join(data_dir, "natemis_n2o.txt"),
        "idtm": 24
    },
)

cscm._run(
    {"output_folder": outdir, "output_prefix": "pams_other"},
    pamset_udm={
        "rlamdo": 7.935264,
        "akapa": 0.33679135961526513,
        "cpi": 0.5513349,
        "W": 1.081163,
        "beto": 0.8246083,
        "threstemp": 6.0,
        "lambda": 0.6086934,
        "mixed": 78.46448,
        "foan": 0.62,
        "foas": 0.82,
        "ebbeta": 0.1,
        "fnso": 0.7532,
        "lm": 39,
        "ldtime": 10,
    },
    pamset_emiconc={
        "qbmb": 0.0,
        "qo3": 0.3,
        "qdirso2": -0.5203740910977426,
        "qindso2": -0.5852790880577513,
        "qbc": 0.2287316893023989,
        "qoc": -0.11778341014690517,
        "ref_yr": 2015,
    },
)



import os
import shutil

import pandas as pd
import pandas.testing as pdt

from ciceroscm import CICEROSCM


def check_output(
    output_dir,
    expected_output_dir,
    update_expected_files=False,
    rtol=1e-2,
    files=["output_em.txt", "output_conc.txt"],
):
    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)
        else:
            res = pd.read_csv(file_to_check, delim_whitespace=True)
            exp = pd.read_csv(file_expected, delim_whitespace=True)
            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def check_output_just_some_lines(
    output_dir,
    expected_output_dir,
    update_expected_files=False,
    rtol=1e-2,
    files=["output_temp.txt", "output_forc.txt"],
    lines=10,
):
    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)
        else:
            res = pd.read_csv(
                file_to_check, delim_whitespace=True, skiprows=range(lines, 352)
            )
            exp = pd.read_csv(
                file_expected, delim_whitespace=True, skiprows=range(lines, 352)
            )
            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def test_ciceroscm_run_pert_forc(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "perturb_forc_file": os.path.join(test_data_dir, "pertforc_test.txt"),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_emiconc={
            "qh2o_ch4": 0.171,
            "qbmb": 0.03,
            "qo3": 0.4,
            "qdirso2": -0.457,
            "qindso2": -0.514,
            "qbc": 0.200,
            "qoc": -0.103,
            "qh2o_ch4": 0.171,
        },
    )

    check_output(
        outdir, os.path.join(test_data_dir, "pert_tests"), files=["output_forc.txt"]
    )
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "perturb_forc_data": pd.read_csv(
                os.path.join(test_data_dir, "pertforc_test.txt")
            ),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_emiconc={
            "qh2o_ch4": 0.171,
            "qbmb": 0.03,
            "qo3": 0.4,
            "qdirso2": -0.457,
            "qindso2": -0.514,
            "qbc": 0.200,
            "qoc": -0.103,
            "qh2o_ch4": 0.171,
        },
    )

    check_output(
        outdir, os.path.join(test_data_dir, "pert_tests"), files=["output_forc.txt"]
    )


def test_ciceroscm_run_pert_emis(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "perturb_em_file": os.path.join(test_data_dir, "pertem_test.txt"),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run({"output_folder": outdir})

    check_output(
        outdir,
        os.path.join(test_data_dir, "pert_tests"),
        files=["output_em.txt"],
    )
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "perturb_em_data": pd.read_csv(
                os.path.join(test_data_dir, "pertem_test.txt"), index_col=None
            ),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run({"output_folder": outdir})

    check_output(
        outdir,
        os.path.join(test_data_dir, "pert_tests"),
        files=["output_em.txt"],
    )



"""
Module for public utilities

Presently mainly methods to make decay functions from arrays
"""

import logging

import numpy as np

LOGGER = logging.getLogger(__name__)


def _check_array_consistency(coeffs, timescales):
    """
    Check array consistency for sets of coefficients and
    decay timescales

    Parameters
    ----------
    coeffs : array
        array, preferably an np.array
    timescales : array
        array, preferably an np.array

    Returns
    -------
    list
        list of two np.ndarrays that are coeffs and timescales
        converted to numpy arrays in case they weren't before
        and checked to be of equal length and with the timescales
        guaranteed to be positive in all errors

    Raises
    ------
    TypeError
        If coeffs or timescales are not arrays
    ValueError
        If any of the timescales are negative or
        if length of timescales and coeffs don't match
    """
    try:
        coeffs = np.array(coeffs)
        timescales = np.array(timescales)
    except TypeError as terr:
        LOGGER.error("Both coefficients and timescales must be 1D arrays")
        raise terr
    if len(coeffs) != len(timescales):
        LOGGER.error("Coefficient array length must be equal to exponent array length")
        raise ValueError(
            "Coefficient array length must be equal to exponent array length"
        )
    if np.any(timescales < 0):
        LOGGER.error("All timescales must be positive numbers")
        raise ValueError("All timescales must be positive numbers")
    return coeffs, timescales


def make_rs_function_from_arrays(rs_coeff, rs_time):
    """
    Make mixed layer pulse response function from arrays of coefficients
    and timescales

    Parameters
    ----------
    rs_coeff : np.ndarray
        array of coefficients to the function
        These should all be postive and sum to less than one
        The constant term will be 1 minus this sum
    rs_time : np.ndarray
        array of decay timescales for each term
        All of these must be postive

    Returns
    -------
    function
       Mixed layer carbon pool decay function based on the coefficient and
       timescale vectors

    Raises
    ------
    ValueError
       If the coefficients are negative, or sum to more than one
    """
    rs_coeff, rs_time = _check_array_consistency(rs_coeff, rs_time)
    coeff0 = 1 - np.sum(rs_coeff)
    if np.any(rs_coeff <= 0):
        LOGGER.error("All coefficients must be positive")
        raise ValueError("All coefficients must be positive")
    if coeff0 < 0:
        raise ValueError(
            "Sum of carbon decay coefficients must be less than or equal to 1"
        )

    return lambda it, idtm: coeff0 + np.dot(rs_coeff, np.exp(-it / rs_time / idtm))


def make_rb_function_from_arrays(rb_coeff, rb_time):
    """
    Make carbon pool decay function from arrays of coefficients
    and timescales

    Parameters
    ----------
    rb_coeff : np.ndarray
        array of coefficients to the function
        All of these must be positive
    rb_time : np.ndarray
        array of decay timescales for each term
        All of these must be postive

    Returns
    -------
    function
       Biotic decay function based on the coefficient and
       timescales
    """
    rb_coeff, rb_time = _check_array_consistency(rb_coeff, rb_time)
    return lambda it, idtm: np.dot(rb_coeff, np.exp(-it / rb_time / idtm))


def find_num_cl_in_hcfc(tracer):
    """
    Find number of Cl-atoms in a CFC or HCFC tracer

    Assuming for now, that no bromines are involved in these
    and Halon names are used for bromine containing CFCs

    Parameters
    ----------
    tracer : str
             Name of tracer, should start with HCFC or CFC
    Returns
    -------
    int
       Number of chlorine atoms in the compound
    """
    numpart = tracer.split("-")[-1]
    if not numpart[-1].isdigit():
        numpart = int(numpart[:-1])
    else:
        numpart = int(numpart)
    print(numpart)
    double_bonds = numpart // 1000
    bonds_total = ((numpart % 1000) // 100) * 2 + 4
    hydrogen = (numpart % 100) // 10 - 1
    fluorine = numpart % 10
    chlorine = bonds_total - 2 * double_bonds - hydrogen - fluorine
    return chlorine


def make_cl_and_br_dictionaries(gases_index):
    """
    Make dictionaries with compound names as keys
    and number of bromine or chlorine as values
    from the compound names in a list
    (from gaspamfile headers)

    Parameters
    ----------
    gases_index : list
                 List with compound names as strings
                 This should be the index of the
                 DataFrame generated from a gaspam_file
    Returns
    -------
    tuple
         The chlorine dictionary followed by the
         bromine dictionary filled in
    """
    chlor_dict = {}
    brom_dict = {}
    for tracer in gases_index:
        if "CFC-" in tracer:
            chlor_dict[tracer] = find_num_cl_in_hcfc(tracer)
        elif tracer[:2] == "H-":
            brom_dict[tracer] = int(tracer[5])
            if int(tracer[4]) > 0:
                chlor_dict[tracer] = int(tracer[4])
        elif "Cl" in tracer:
            if len(tracer.split("Cl")[-1]) > 0 and tracer.split("Cl")[-1][0].isdigit():
                chlor_dict[tracer] = int(tracer.split("Cl")[-1][0])
            else:
                chlor_dict[tracer] = 1
        elif "Br" in tracer:
            if len(tracer.split("Br")[-1]) > 0 and tracer.split("Br")[-1][0].isdigit():
                brom_dict[tracer] = int(tracer.split("Br")[-1][0])
            else:
                brom_dict[tracer] = 1

    return chlor_dict, brom_dict



#Plot radiative forcing  in the CICERO SCM
#and compare to IPCC AR6 forcing timeseries.
#https://github.com/chrisroadmap/ar6/blob/main/data_output/AR6_ERF_1750-2019.csv
#https://raw.githubusercontent.com/chrisroadmap/ar6/main/data_output/AR6_ERF_1750-2019.csv
#Can add 90% CI as well.


import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

plt.rcParams['font.size'] = 4

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_rf=pd.read_csv(outdir+'/output_forc.txt', sep='\t', index_col=0)
#print(df_rf)


#Read ipcc AR6 forcing timeseries
df_rf_ar6 = pd.read_csv('data_compare/AR6_ERF_1750-2019.csv',sep=',',header=0,index_col=0)

#print(df_rf_ar6.columns)
#Components given in IPCC
#'co2', 'ch4', 'n2o', 'other_wmghg', 'o3', 'h2o_stratospheric',
#       'contrails', 'aerosol-radiation_interactions',
#       'aerosol-cloud_interactions', 'bc_on_snow', 'land_use', 'volcanic',
#       'solar', 'nonco2_wmghg', 'aerosol', 'chapter2_other_anthro',
#       'total_anthropogenic', 'total_natural', 'total'

complist_ar6 = ['co2', 'ch4', 'n2o', 'other_wmghg',
                'o3','h2o_stratospheric','land_use','aerosol',
                'total_anthropogenic']


fig, axs = plt.subplots(nrows=3, ncols=3,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Radiative Forcing')
for i,comp in enumerate(complist_ar6):
    print(i)
    print(comp)
    df_rf_ar6[comp].plot(ylabel='RF [Wm$^{-2}$ ]',
                         ax=axs[i],color='black',label='IPCC AR6')
    if comp == 'co2':
        df_rf['CO2'].plot(ax=axs[i],label=scen)
    elif comp == 'ch4':
        df_rf['CH4'].plot(ax=axs[i],label=scen)
    elif comp == 'n2o':
        df_rf['N2O'].plot(ax=axs[i],label=scen)
    elif comp == 'other_wmghg':
        complist_other=['CFC-11', 'CFC-12', 'CFC-113', 'CFC-114',
                        'CFC-115', 'CH3Br', 'CCl4', 'CH3CCl3', 'HCFC-22', 'HCFC-141b',
                        'HCFC-142b', 
                        'C2F6', 'C6F14', 'CF4', 'SF6','HCFC-123',          
                        'H-1211', 'H-1301', 'H-2402','HFC125','HFC134a',
                        'HFC143a','HFC227ea', 'HFC23', 'HFC245fa','HFC32',
                        'HFC4310mee']
        df_rf['other_wmghg'] = df_rf[complist_other].sum(axis=1, skipna=True)
        #print(df_rf['other_wmghg'])
        df_rf['other_wmghg'].plot(ax=axs[i],label=scen)
    elif comp == 'o3':
        print(df_rf.columns)
        df_rf['o3'] = df_rf['TROP_O3']+df_rf['STRAT_O3'] 
        df_rf['o3'].plot(ax=axs[i],label=scen)
    elif comp == 'h2o_stratospheric':
        df_rf['STRAT_H2O'].plot(ax=axs[i],label=scen)
    elif comp == 'land_use':
        df_rf['LANDUSE'].plot(ax=axs[i],label=scen)
    elif comp == 'aerosol':
        complist_aerosols = ['SO2','SO4_IND','BMB_AEROS_BC',
                             'BMB_AEROS_OC','BMB_AEROS', 'BC', 'OC']
        df_rf['aerosol'] = df_rf[complist_aerosols].sum(axis=1, skipna=True)
        df_rf['aerosol'].plot(ax=axs[i],label=scen)
    elif comp == 'total_anthropogenic':
        df_rf['Total_forcing'].plot(ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].axhline(y=0,color='k',linestyle=':',linewidth=0.5)
    

    
fig, axs = plt.subplots(nrows=1, ncols=1,sharex=True,figsize=(6,6))
fig.suptitle('CICERO SCM simulation, Aerosol Radiative Forcing')


comp = 'aerosol-radiation_interactions'
df_rf_ar6[comp].plot(ylabel='RF [Wm$^{-2}$ ]',
                     ax=axs,color='green',label='IPCC AR6 '+comp)

comp = 'aerosol-cloud_interactions'
df_rf_ar6[comp].plot(ylabel='RF [Wm$^{-2}$ ]',
                     ax=axs,color='blue',label='IPCC AR6 '+comp)

complist_aerosols_direct = ['SO2','BMB_AEROS_BC',
                            'BMB_AEROS_OC','BMB_AEROS', 'BC', 'OC']
df_rf['aerosol_direct'] = df_rf[complist_aerosols_direct].sum(axis=1, skipna=True)
df_rf['aerosol_direct'].plot(ax=axs,color='purple',label='SCM ' + scen + ' sum aerosols direct')
df_rf['SO4_IND'].plot(ax=axs,color='orange',label='SCM ' + scen + ' SO4 indirect')



axs.legend()

plt.show()
exit()



"""
Module to simplify doing a run over a configuration distribution
"""

import json
import logging
import os

from .cscmparwrapper import run_ciceroscm_parallel

LOGGER = logging.getLogger(__name__)


class DistributionRun:
    """
    Class that defines the distribution run
    """

    def __init__(
        self, distro_config, json_file_name="no_file", indexer_pre="", numvalues=100
    ):
        """
        Intialise DistributionRun

        Parameters
        ----------
        distro_config: obj
              Instance of the _ConfigDistro class that defines the
              distribution to be run, in this case the configurations
              will be generated from numvalues of this distribution
              If the file json_file_name exists, this will be ignored
        json_file_name: str
              path to json_file with configurations if the file
              exists, configurations will be read from here
        indexer_pre: str
              prefix to put on the Index for the samples
        numvalues: int
              number of configurations to generate, ignored if
              samples_from_json = True
        """
        if os.path.exists(json_file_name):
            with open(json_file_name, "r", encoding="utf-8") as rfile:
                self.cfgs = json.load(rfile)
        else:
            self.cfgs = distro_config.make_config_list(
                numvalues, indexer_pre=indexer_pre
            )

    def run_over_distribution(self, scendata, output_vars, max_workers=4):
        """
        Run over distribution

        scendata: list or dict
            list of dictionaries with the scenarios to run over
            or dictionary of single scenario
        output_vars: list
            list of variables to output
        max_workers: int
            maximum number of parallel node workers to employ
            for the run
        """
        if isinstance(scendata, dict):
            scendata = [scendata]
        results = run_ciceroscm_parallel(scendata, self.cfgs, output_vars, max_workers)
        return results

    def write_configs_to_json(self, json_file_name):
        """
        Write configs to json

        Parameters
        ----------
        json_file_name: str
            path for json file to write configurations in
        """
        with open(json_file_name, "w", encoding="utf-8") as wfile:
            json.dump(self.cfgs, wfile)



import os.path

import pytest

# from ciceroscm import CICEROSCM

TEST_DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test-data")


@pytest.fixture(scope="session")
def test_data_dir():
    return TEST_DATA_DIR



"""
Two functions to make output plots, saves the plots to a sub-folder in the output-folder
"""

import os

import matplotlib.pyplot as plt
import numpy as np


def fix_plot(axs, ylabel, title, legend=False):
    """
    Add labels to ohc, rib and temp plot

    Adding Year as xlabel, and a list of ylabels
    and list of titles to each of a list of axes
    If choosen add label
    Used typically  to annotate ohc rib and temp plot

    Parameters
    ----------
    axs : list
       list of plot axes
    ylabel : list
          list of ylabels for each axis
    title : list
          list of ylabels for each axis
    legend : bool
          Whether or not to add a legend to the
          plot axes
    """
    axs = axs.flatten()
    for i, ylab in enumerate(ylabel):
        axs[i].set_xlabel("Year")
        axs[i].set_ylabel(ylab)
        axs[i].set_title(title[i], fontsize=11, fontweight="bold")
        axs[i].set_title(f"{chr(i+97)})", fontsize=11, loc="left")
        if legend:
            axs[i].legend()


def plot_output1(pamset, results, nystart, nyend):
    """
    Plot ohc, rib and temp output

    Parameters
    ----------
    pamset : dict
          pamset possibly containing output_folder path
    results : dict
           Dictionary with results to plot
    nystart : int
           Startyear of dataset
    nyend : int
         Endyear of dataset
    """
    if "output_folder" in pamset:
        outdir = os.path.join(os.getcwd(), pamset["output_folder"])
    else:
        outdir = os.path.join(os.getcwd(), "output")
    if os.path.exists(outdir):
        plotdir = os.path.join(outdir, "plots")
        if not os.path.exists(plotdir):
            os.makedirs(plotdir)
    else:
        return
    #    if not os.path.exists("plots"):
    #        os.makedirs("plots")
    indices = np.arange(nystart, nyend + 1)
    fig, axs = plt.subplots(nrows=1, ncols=2, sharex=True, sharey=True, figsize=(12, 6))
    axs[0].plot(indices, results["OHC700"])
    axs[1].plot(indices, results["OHCTOT"])
    fix_plot(axs, ["OHC [$10^{22}$J]", "OHC [$10^{22}$J]"], ["OHC700", "OHCTOT"])
    fig.suptitle("CICERO SCM simulation, Ocean heat content")
    axs[1].yaxis.set_tick_params(labelbottom=True)
    plt.savefig(os.path.join(plotdir, "ohc.png"))
    fig, axs = plt.subplots(nrows=1, ncols=2, sharex=True, figsize=(12, 6))
    axs[0].plot(indices, results["RIB_glob"], label="RIB_glob")
    axs[1].plot(indices, results["RIB_glob"], label="RIB_glob")
    axs[1].plot(indices, results["RIB_N"], label="RIB_N", linestyle=":")
    axs[1].plot(indices, results["RIB_S"], label="RIB_S", linestyle="--")
    fix_plot(
        axs,
        [r"W $\mathrm{m}^{-2}$", r"W $\mathrm{m}^{-2}$"],
        ["RIB_glob", "RIB_glob"],
        legend=True,
    )
    fig.suptitle("CICERO SCM simulation, RIB")
    plt.savefig(os.path.join(plotdir, "rib.png"))
    fig, axs = plt.subplots(nrows=1, ncols=2, sharex=True, figsize=(14, 6))

    for comp in ["dT_glob", "dT_glob_air", "dT_glob_sea"]:
        colours = {"glob": "tab:green", "air": "tab:orange", "sea": "tab:blue"}
        axs[0].plot(
            indices,
            results[comp],
            label=comp,
            color=colours[comp.rsplit("_", maxsplit=1)[-1]],
        )
    for comp in [
        "dT_NH",
        "dT_NH_air",
        "dT_NH_sea",
        "dT_SH",
        "dT_SH_air",
        "dT_SHsea",
    ]:
        if "NH" in comp:
            colours = {"NH": "tab:green", "air": "tab:orange", "sea": "tab:blue"}
            axs[1].plot(
                indices,
                results[comp],
                label=comp,
                linestyle=":",
                color=colours[comp.rsplit("_", maxsplit=1)[-1]],
            )
        else:
            colours = {"SH": "tab:green", "air": "tab:orange", "SHsea": "tab:blue"}
            axs[1].plot(
                indices,
                results[comp],
                label=comp,
                linestyle="--",
                color=colours[comp.rsplit("_", maxsplit=1)[-1]],
            )
    fix_plot(
        axs,
        ["K", "K"],
        ["Temperature change global", "Temperature change global and hemispheric"],
        legend=True,
    )
    plt.savefig(os.path.join(plotdir, "temp.png"))


def plot_output2(
    var, df_in, outdir, unit=None
):  # pylint: disable=too-many-locals, too-many-branches
    """
    Plot concentration, emission and forcing

    Parameters
    ----------
    var : str
       Denoting whether data is concentrations (conc),
       emissions (emis) or forcing (forc)
    df_in : pd.dataframe
         Dataframe to plot
    outdir : str
          path of where to send plot
    unit : pd.dataframe
        dataframe possibly containing units
    """
    plotdir = os.path.join(outdir, "plots")
    if var == "emis":
        df_in = df_in.drop(labels=unit[unit.values == "X"].index.tolist(), axis=1)
    elif var == "conc":
        df_in = df_in.drop(labels=unit[unit.values == "-"].index.tolist(), axis=1)
    elif var == "forc":
        df_in = df_in.drop(
            labels=["NOx", "CO", "NMVOC", "NH3", "BMB_AEROS_OC", "BMB_AEROS_BC"], axis=1
        )
    years = df_in["Year"]
    comps = df_in.columns[1:]
    for c, comp in enumerate(comps):
        if c in [0, 16, 32]:
            maxrows = 4
            if c == 32:
                maxrows = int(np.ceil((len(comps) - 32) / 4))
            fig, axs = plt.subplots(
                nrows=maxrows, ncols=4, sharex=True, figsize=(15, 10)
            )
            if var == "forc":
                title = (
                    "CICERO SCM simulation, Radiative Forcing "
                    + str(int(c / 16 + 1))
                    + " of 3"
                )
                fname = "forc_" + str(int(c / 16 + 1)) + ".png"
                fig.supylabel("RF [Wm$^{-2}$]")
            elif var == "emis":
                title = (
                    "CICERO SCM simulation, Emissions " + str(int(c / 16 + 1)) + " of 3"
                )
                fname = "em_" + str(int(c / 16 + 1)) + ".png"
            else:
                title = (
                    "CICERO SCM simulation, Concentrations "
                    + str(int(c / 16 + 1))
                    + " of 2"
                )
                fname = "conc_" + str(int(c / 16 + 1)) + ".png"
            fig.suptitle(title)
            i = 0
            j = 0
        axs[i, j].plot(years, df_in[comp])
        if comp != "Total_forcing":
            axs[i, j].set_title(comp, fontsize=11, fontweight="bold")
            axs[i, j].set_title(f"{chr(i*4+j+97)})", fontsize=11, loc="left")
        else:
            axs[i, j].set_title("Total anthropogenic", fontsize=11, fontweight="bold")
            axs[i, j].set_title(f"{chr(i*4+j+97)})", fontsize=11, loc="left")
        axs[i, j].xaxis.set_tick_params(labelbottom=True)
        if var != "forc":
            axs[i, j].set_ylim(ymin=0)
            axs[i, j].set_ylabel("[" + unit.loc[comp] + "]")
        if c in [15, 31, len(comps) - 1]:
            if c == len(comps) - 1:
                for j_left in range(j + 1, 4):
                    fig.delaxes(axs[i, j_left])
            fig.tight_layout(pad=4, h_pad=2, w_pad=1)
            fig.supxlabel("Year")
            fig.savefig(os.path.join(plotdir, fname))
        j += 1
        if j == 4:
            j = 0
            i += 1



import os
import shutil

import numpy as np
import pandas as pd
import pandas.testing as pdt

from ciceroscm import CICEROSCM, input_handler


def check_output(
    output_dir,
    expected_output_dir,
    update_expected_files=False,
    rtol=1e-2,
    files=["output_em.txt", "output_conc.txt"],
):
    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)
        print(filename)
        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)
        else:
            res = pd.read_csv(file_to_check, delim_whitespace=True)
            exp = pd.read_csv(file_expected, delim_whitespace=True)

            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def check_output_just_some_lines(
    output_dir,
    expected_output_dir,
    update_expected_files=False,
    rtol=1e-2,
    files=["output_temp.txt", "output_forc.txt"],
    lines=10,
):
    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)
        else:
            res = pd.read_csv(
                file_to_check, delim_whitespace=True, skiprows=range(lines, 352)
            )
            exp = pd.read_csv(
                file_expected, delim_whitespace=True, skiprows=range(lines, 352)
            )
            pdt.assert_index_equal(res.index, exp.index)

            pdt.assert_frame_equal(
                res.T,
                exp.T,
                check_like=True,
                rtol=rtol,
            )


def test_ciceroscm_run_emi(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            # "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "gaspam_file": os.path.join(test_data_dir, "gases_vupdate_2022_AR6.txt"),
            "nyend": 2100,
            "nystart": 1750,
            "emstart": 1850,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 15.1,
            "akapa": 0.657,
            "cpi": 0.208,
            "W": 2.2,
            "beto": 6.9,
            "lambda": 0.606,
            "mixed": 107.0,
        },
        pamset_emiconc={
            "qbmb": 0.0,
            "qo3": 0.5,
            "qdirso2": -0.3701,
            "qindso2": -0.4163,
            "qbc": 0.163,
            "qoc": -0.084,
            "qh2o_ch4": 0.171,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "ssp245_emis"))
    check_output_just_some_lines(
        outdir,
        os.path.join(test_data_dir, "ssp245_emis"),
        files=["output_forc.txt"],
        lines=14,
    )
    check_output_just_some_lines(
        outdir,
        os.path.join(test_data_dir, "ssp245_emis"),
        files=["output_temp.txt"],
        lines=16,
    )


def test_ciceroscm_short_run(tmpdir, test_data_dir):
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:
    nystart = 1900
    nyend = 2050
    emstart = 1950
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nystart": nystart,
            "emstart": emstart,
            "nyend": nyend,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "idtm": 24,
            "sunvolc": 1,
            "rf_luc_data": pd.read_csv(
                os.path.join(test_data_dir, "land_use_erf_ar6.txt"),
                header=None,
                skiprows=1,
                index_col=0,
            ),
            "rf_sun_data": pd.read_csv(
                os.path.join(test_data_dir, "solar_erf_ar6.txt"),
                header=None,
                skiprows=1,
                index_col=0,
            ),
            "rf_volc_data": pd.read_csv(
                os.path.join(test_data_dir, "volcanic_erf_ar6.txt"),
                header=None,
                skiprows=1,
                index_col=0,
            ),
        },
    )

    cscm._run({"output_folder": outdir})

    file_results = os.path.join(outdir, "output_em.txt")
    exp_index = np.arange(nystart, nyend + 1)
    res = pd.read_csv(file_results, delim_whitespace=True)
    np.testing.assert_equal(res.Year.to_numpy(), exp_index)

    cscm._run({"results_as_dict": True})
    expected_keys = [
        "emissions",
        "concentrations",
        "forcing",
        "OHC700",
        "OHCTOT",
        "RIB_glob",
        "RIB_N",
        "RIB_S",
        "dT_glob",
        "dT_NH",
        "dT_SH",
        "dT_glob_air",
        "dT_NH_air",
        "dT_SH_air",
        "dT_glob_sea",
        "dT_NH_sea",
        "dT_SHsea",
        "Solar_forcing",
        "Volcanic_forcing_NH",
        "Volcanic_forcing_SH",
    ]
    for key in expected_keys:
        assert key in cscm.results
    # Put this in again, find out what is happening with CF4
    # check_output(
    #    outdir_save,
    #    os.path.join(test_data_dir, "ssp245_emis"),
    #    files=["output_temp.txt"],
    #    rtol=1.0,
    # )

    # check_output(
    #    outdir_save,
    #    os.path.join(test_data_dir, "ssp245_emis"),
    #    files=["output_forc.txt"],
    #    rtol=0.1,
    # )


def test_ciceroscm_run_conc(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "conc_run": True,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "sunvolc": 1,
        },
    )
    outdir = str(tmpdir)
    # outdir_save = os.path.join(os.getcwd(), "output")

    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_emiconc={
            "qh2o_ch4": 0.171,
            "qbmb": 0.03,
            "qo3": 0.4,
            "qdirso2": -0.457,
            "qindso2": -0.514,
            "qbc": 0.200,
            "qoc": -0.103,
            "qh2o_ch4": 0.171,
        },
        pamset_udm={
            "rlamdo": 16.0,
            "akapa": 0.634,
            "cpi": 0.4,
            "W": 4.0,
            "beto": 3.5,
            "lambda": 0.540,
            "mixed": 60.0,
        },
    )

    check_output(
        outdir,
        os.path.join(test_data_dir, "ssp245_conc"),
        files=["output_conc.txt", "output_em.txt", "output_forc.txt", "output_ohc.txt"],
    )


def test_run_with_data_not_files(tmpdir, test_data_dir):
    ih = input_handler.InputHandler({"nystart": 1750, "nyend": 2100, "emstart": 1850})
    cscm = CICEROSCM(
        {
            "gaspam_data": input_handler.read_components(
                os.path.join(test_data_dir, "gases_vupdate_2022_AR6.txt")
            ),
            "nyend": 2100,
            "nystart": 1750,
            "emstart": 1850,
            "concentrations_data": input_handler.read_inputfile(
                os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"), True, 1750, 2100
            ),
            "emissions_data": ih.read_emissions(
                os.path.join(test_data_dir, "ssp245_em_RCMIP.txt")
            ),
            "nat_ch4_data": input_handler.read_natural_emissions(
                os.path.join(test_data_dir, "natemis_ch4.txt"), "CH4"
            ),
            "nat_n2o_data": input_handler.read_natural_emissions(
                os.path.join(test_data_dir, "natemis_n2o.txt"), "N2O"
            ),
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:

    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 15.1,
            "akapa": 0.657,
            "cpi": 0.208,
            "W": 2.2,
            "beto": 6.9,
            "lambda": 0.606,
            "mixed": 107.0,
        },
        pamset_emiconc={
            "qbmb": 0.0,
            "qo3": 0.5,
            "qdirso2": -0.3701,
            "qindso2": -0.4163,
            "qbc": 0.163,
            "qoc": -0.084,
            "qh2o_ch4": 0.171,
        },
    )

    check_output(outdir, os.path.join(test_data_dir, "ssp245_emis"))
    check_output_just_some_lines(
        outdir,
        os.path.join(test_data_dir, "ssp245_emis"),
        files=["output_forc.txt"],
        lines=14,
    )
    check_output_just_some_lines(
        outdir,
        os.path.join(test_data_dir, "ssp245_emis"),
        files=["output_temp.txt"],
        lines=16,
    )


def test_ciceroscm_just_one(tmpdir, test_data_dir):
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            "nyend": 2100,
            "conc_run": True,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
        },
    )
    # outdir = str(tmpdir)

    # One year forcing:

    cscm._run(
        {"results_as_dict": True},
        pamset_emiconc={"qh2o_ch4": 0.171, "just_one": "CO2"},
    )
    assert cscm.results["forcing"]["CO2"].equals(
        cscm.results["forcing"]["Total_forcing"]
    )

    cscm2 = CICEROSCM(
        {"nyend": 2100, "forc_data": cscm.results["forcing"]["CO2"].to_numpy()}
    )

    cscm2._run({"results_as_dict": True})

    for key in cscm2.results:
        assert np.array_equal(cscm2.results[key], cscm.results[key])



import os

import numpy as np

from ciceroscm import input_handler, perturbations


def test_emission_perturbation(test_data_dir):
    emissions_file = os.path.join(test_data_dir, "ssp245_em_RCMIP.txt")
    pert_file = os.path.join(test_data_dir, "pertem_test.txt")
    i_handler = input_handler.InputHandler(
        {"emissions_file": emissions_file, "perturb_em_file": pert_file}
    )
    em_df = i_handler.get_data("emissions")
    em_df_change = perturbations.perturb_emissions(i_handler, em_df)
    print(em_df["CO2_FF"][2000])
    print(em_df_change["CO2_FF"][2000])
    em_df_old = i_handler.get_data("emissions")
    diff = em_df_old.sub(em_df_change)
    print(diff["CO2_FF"][2000])
    assert diff["CO2_FF"][1999] == 0.0
    assert diff["CO2_FF"][2000] == 4.0
    assert diff["CO2_FF"][2001] == 4.0
    assert diff["CO2_FF"][2002] == 4.0
    assert diff["CH4"][1864] == 0.0
    assert diff["CH4"][2000] == 4.0
    assert diff["CH4"][2001] == 4.0
    assert diff["CH4"][2002] == 5.0
    assert diff["N2O"][2078] == 0.0
    assert diff["N2O"][2000] == -8.0
    assert diff["N2O"][2001] == 4.0
    assert diff["N2O"][2002] == 4.0


def test_make_forcing_perturbations_dataset(test_data_dir):
    pert_file = os.path.join(test_data_dir, "pertforc_test.txt")
    i_handler = input_handler.InputHandler({"perturb_forc_file": pert_file})
    forc_pert = perturbations.ForcingPerturbation(i_handler, 1999)
    assert forc_pert.check_if_year_in_pert(2000)
    assert not forc_pert.check_if_year_in_pert(1983)
    assert forc_pert.check_if_compound_in_pert("OTHER")
    assert not forc_pert.check_if_compound_in_pert("CO2")
    test = np.array([38.0, 42.0, 48.0, 48.0])
    forc = {"CO2": test, "OTHER": [0, 0, 0, 0]}

    for year in range(1999, 2003):
        totforc = forc["CO2"][year - 1999]
        totforc, forc_nh, forc_sh, forc = forc_pert.add_forcing_pert(
            totforc, totforc, totforc, forc, year
        )
        if year == 1999:
            expected = test[0]
        else:
            expected = test[year - 1999] - 4.0
        assert totforc == expected
    assert np.array_equal(forc["OTHER"], [0, -4.0, -4.0, -4.0])



#Plot radiative forcing all components in the CICERO SCM
#Not all components have radiative forcing output.

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

plt.rcParams['font.size'] = 4

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_rf=pd.read_csv(outdir+'/output_forc.txt', sep='\t', index_col=0)
#print(df_rf)

#Read components, to get the units:
inputdir = '/div/no-backup/users/ragnhibs/ciceroscm/tests/test-data/'
df_comp =pd.read_csv(inputdir + 'gases_v1RCMIP.txt', sep='\t', index_col=0)
print(df_comp.loc['CO2'])

antcomp = len(df_rf.columns)
print(antcomp)
#print(len(df_comp.index))


#Plot first 16 components:
complist = np.arange(0,16)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Radiative Forcing 1 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_rf.columns[c]
    print(comp)
    df_rf[comp].plot(ylabel='RF [Wm$^{-2}$ ]',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].axhline(y=0,color='k',linestyle=':',linewidth=0.5)
    
#Plot next 16 components:
complist = np.arange(16,32)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Radiative Forcing 2 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_rf.columns[c]
    print(comp)
    df_rf[comp].plot(ylabel='RF [Wm$^{-2}$]',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].axhline(y=0,color='k',linestyle=':',linewidth=0.5)
#Plot rest of components:
complist = np.arange(32,antcomp)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Radiative Forcing 3 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_rf.columns[c]
    print(comp)
    df_rf[comp].plot(ylabel='RF [Wm$^{-2}$]',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].axhline(y=0,color='k',linestyle=':',linewidth=0.5)

plt.show()
exit()



"""
Test that all of our modules can be imported
Thanks https://stackoverflow.com/a/25562415/10473080
and openscm-runner
"""
import importlib
import os.path
import pkgutil

import ciceroscm

def import_submodules(package_name):
    package = importlib.import_module(package_name)

    for _, name, is_pkg in pkgutil.walk_packages(package.__path__):
        full_name = package.__name__ + "." + name
        print(full_name)
        importlib.import_module(full_name)
        if is_pkg:
            import_submodules(full_name)


import_submodules("ciceroscm")

# make sure input data etc. are included
ciceroscm_root = os.path.dirname(ciceroscm.__file__)
assert os.path.isfile(
    os.path.join(
        ciceroscm_root, "default_data/IPCC_LUCalbedo.txt"
    )
)
assert os.path.isfile(
    os.path.join(
        ciceroscm_root, "default_data/solar_IPCC.txt"
    )
)
assert os.path.isfile(
    os.path.join(
        ciceroscm_root, "default_data/meanVOLCmnd_ipcc_NH.txt"
    )
)

print(ciceroscm.__version__)



#To be fixed in scm-code: dT_SHsea

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

plt.rcParams['font.size'] = 4

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_temp=pd.read_csv(outdir+'/output_temp.txt', sep='\t', index_col=0)
print(df_temp)

antcomp = len(df_temp.columns)
print(antcomp)
#print(len(df_comp.index))


fig, axs = plt.subplots(nrows=1, ncols=3,sharex=True,figsize=(12,6))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Temperature')


i=0
comp = 'dT_glob'
df_temp[comp].plot(ylabel='Temp. [C$^\circ$ ]',ax=axs[i],label=scen)

i=1
df_temp['dT_glob'].plot(ylabel='Temp. [C$^\circ$ ]',ax=axs[i],label='dT_glob')
df_temp['dT_glob_air'].plot(ax=axs[i],label='dT_glob_air')
df_temp['dT_glob_sea'].plot(ax=axs[i],label='dT_glob_sea')

i=2
df_temp['dT_glob'].plot(ylabel='Temp. [C$^\circ$ ]',ax=axs[i],label='dT_glob')
df_temp['dT_glob_air'].plot(ax=axs[i],label='dT_glob_air')
df_temp['dT_glob_sea'].plot(ax=axs[i],label='dT_glob_sea')

df_temp['dT_NH'].plot(linestyle=':',ax=axs[i],label='dT_NH')
df_temp['dT_NH_air'].plot(ax=axs[i],linestyle=':',label='dT_NH_air')
df_temp['dT_NH_sea'].plot(ax=axs[i],linestyle=':',label='dT_NH_sea')

df_temp['dT_SH'].plot(linestyle='--',ax=axs[i],label='dT_SH')
df_temp['dT_SH_air'].plot(ax=axs[i],linestyle='--',label='dT_SH_air')
df_temp['dT_SHsea'].plot(ax=axs[i],linestyle='--',label='dT_SHsea')

axs[0].legend()
axs[1].legend()
axs[2].legend()

axs[0].axhline(y=0,color='k',linestyle=':',linewidth=0.5)


fig, axs = plt.subplots(nrows=1, ncols=2,sharex=True,figsize=(12,6))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, SLR (!!!!!need to be revised-checked!!!!)')
df_temp['dSL(m)'].plot(ylabel='Sea level rise [m]',ax=axs[0],label=scen)

df_temp['dSL(m)'].plot(ylabel='Sea level rise [m]',ax=axs[1],label='dSL(m)')
df_temp['dSL_thermal(m)'].plot(ylabel='Sea level rise [m]',
                               ax=axs[1],label='dSL_thermal(m)')
df_temp['dSL_ice(m)'].plot(ylabel='Sea level rise [m]',
                           ax=axs[1],label='dSL_ice(m)')

axs[0].legend()
axs[1].legend()

plt.show()



#Plot emissions all components in the CICERO SCM
#If unit is 'X', no emissions is provided, but component in output file.

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

plt.rcParams['font.size'] = 4

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_em=pd.read_csv(outdir+'/output_em.txt', sep='\t', index_col=0)
#print(df_em)

#Read components, to get the units:
inputdir = '/div/no-backup/users/ragnhibs/ciceroscm/tests/test-data/'
df_comp =pd.read_csv(inputdir + 'gases_v1RCMIP.txt', sep='\t', index_col=0)
#print(df_comp.loc['CO2']['EM_UNIT'])

antcomp = len(df_em.columns)
print(antcomp)
#print(len(df_comp.index))


#Plot first 16 components:
complist = np.arange(0,16)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Emissions 1 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_em.columns[c]
    print(comp)
    df_em[comp].plot(ylabel='Emissions ['+df_comp.loc[comp]['EM_UNIT']+']',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].set_ylim(bottom=0)

    
#Plot next 16 components:
complist = np.arange(16,32)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Emissions 2 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_em.columns[c]
    print(comp)
    df_em[comp].plot(ylabel='Emissions ['+df_comp.loc[comp]['EM_UNIT']+']',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].set_ylim(bottom=0)
    
#Plot rest of components:
complist = np.arange(32,antcomp)
fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,20))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Emissions 3 of 3')
for i,c in enumerate(complist):
    print(i)
    comp = df_em.columns[c]
    print(comp)
    df_em[comp].plot(ylabel='Emissions ['+df_comp.loc[comp]['EM_UNIT']+']',ax=axs[i],label=scen)
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].set_ylim(bottom=0)

plt.show()
exit()



import os
import shutil
import sys
import cProfile, pstats, io
import pandas as pd

# Adding location of source code to system path
# os.path.dirname(__file__) gives the directory of
# current file. Put in updated path if running script from elsewhere
# os.path joins all the folders of a path together in a
# system independent way (i.e. will work equally well on Windows, linux etc)
sys.path.append(os.path.join(os.path.dirname(__file__), "../", "src"))

from ciceroscm import CICEROSCM

data_dir = os.path.join(os.path.dirname(__file__), "../", "tests", "test-data")
#os.getcwd() gets the path of where you are running from
outdir = os.path.join(os.getcwd(), "output_test")

# Starting profiler before starting run
pr = cProfile.Profile()
pr.enable()

cscm = CICEROSCM(
    {
        "gaspamfile": os.path.join(data_dir, "gases_v1RCMIP.txt"),
        "sunvolc": 1,
        "nyend": 2100,
        "forc_file": os.path.join(data_dir, "CO2_1pros.txt"),
    },
)

cscm._run({"output_folder": outdir})

# Stop counting with profiler
pr.disable()

# Sorting and printing profiler results
s = io.StringIO()
sortby = 'cumulative'
ps = pstats.Stats(pr,stream=s).sort_stats(sortby)
ps.print_stats()
print(s.getvalue())



#Compare CSCM results to
#von Schuckmann et al. Heat stored in the Earth system:
#Where does the energy #go? Earth Syst. Sci. Data, 12,2013-2041,
#10.5194/essd-12-2013-2020, 2020.

#The python environment for cicero scm did not work for nc-file.
#Read file prepared for UTRICS.

#Can add uncertainties and other timeseries as well.

#For comparing with observations, important to add volcanoes as well.

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import xarray as xr


#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_ohc=pd.read_csv(outdir+'/output_ohc.txt', sep='\t', index_col=0)
print(df_ohc)

#Did not work with the CICERO SCM environment
#filename =  'data_compare/GCOS_all_heat_content_1960-2018_ZJ_v22062020.nc'
#gcos = xr.open_dataset(filename)
#print(gcos)

baseyears = [1961,1990]
baseyears_str = str(baseyears[0]) +'-' + str(baseyears[1])


df_gcos = pd.read_csv('/div/qbo/utrics/Observations/OHC/GCOS20_vonSchuckmann/GCOS_ohc.csv',index_col=0)
df_gcos = df_gcos*0.1 #ZJ -> 10^22 J
df_gcos = df_gcos - df_gcos.loc[baseyears[0]:baseyears[1]].mean()
print(df_gcos)


fig, axs = plt.subplots(nrows=1, ncols=2,sharex=True,figsize=(12,6))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Ocean heat content')


comp = 'OHC700'
df_ohc['anomaly 0-700'] = df_ohc[comp] - df_ohc[comp].loc[baseyears[0]:baseyears[1]].mean()
df_ohc['anomaly 0-700'].plot(ylabel='OHC [10^22 J]',ax=axs[0],label=scen)
df_gcos['ohc_0_700m'].plot(ax=axs[0],color='black',label='GCOS')

axs[0].set_title(comp)


comp = 'OHCTOT'
df_ohc['anomaly TOT'] = df_ohc[comp] - df_ohc[comp].loc[baseyears[0]:baseyears[1]].mean()
df_ohc['anomaly TOT'].plot(ylabel='OHC [10^22 J]',ax=axs[1],label=scen)
df_gcos['tot'] = df_gcos['ohc_0_700m'] + df_gcos['ohc_700_2000m']+ df_gcos['ohc_below_2000m']
df_gcos['tot'].plot(ax=axs[1],color='black',label='GCOS')

axs[1].set_title(comp)

axs[0].legend()
axs[1].legend()

plt.show()



exit()



#Can add other temperature series than HadCRUT.

from io import StringIO
import pandas as pd
import requests
import matplotlib.pyplot as plt
import numpy as np

plt.rcParams['font.size'] = 8

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_temp=pd.read_csv(outdir+'/output_temp.txt', sep='\t', index_col=0)
print(df_temp)

antcomp = len(df_temp.columns)
print(antcomp)
#print(len(df_comp.index))

#Read HADCRUT temp
headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.76 Safari/537.36'}

url_hadcrut = "https://www.metoffice.gov.uk/hadobs/hadcrut5/data/current/analysis/diagnostics/HadCRUT.5.0.1.0.analysis.summary_series.global.annual.csv"
s=requests.get(url_hadcrut, headers= headers).text
hadcrut=pd.read_csv(StringIO(s),index_col=0)

#Possible to add several other dataseries as well.

#Same base period as HadCRUT
baseyears = [1961,1990]
baseyears_str = str(baseyears[0]) +'-' + str(baseyears[1])



fig, axs = plt.subplots(nrows=1, ncols=1,sharex=True,figsize=(8,6))

fig.suptitle('CICERO SCM simulation, Temperature')

df_temp['anomaly'] = df_temp['dT_glob']- df_temp['dT_glob'].loc[baseyears[0]:baseyears[1]].mean()

df_temp['anomaly'].plot(ax=axs,label=scen)
hadcrut['Anomaly (deg C)'].plot(ax=axs,color='black',label='HadCRUT')

axs.set_ylabel('GMST relative to ' + baseyears_str + '[$^\circ$C]')

axs.legend()

plt.show()



import os
import sys

# Adding location of source code to system path
# os.path.dirname(__file__) gives the directory of
# current file. Put in updated path if running script from elsewhere
# os.path joins all the folders of a path together in a
# system independent way (i.e. will work equally well on Windows, linux etc)
sys.path.append(os.path.join(os.path.dirname(__file__), "../", "src"))

from ciceroscm import CICEROSCM

# Not system independent, can only be run on amoc or qbo
# If running on different system, this must be changed
# to where you have ssp input files
data_dir = os.path.join(os.path.dirname(__file__), "../", "tests", "test-data")
input_dir = "/div/amoc/CSCM/SCM_Linux_v2019/RCMIP/input/"

outdir = os.path.join(os.getcwd(), "./output_test")

ssps = ["119", "126", "245", "245methane", "370-lowNTCF", "370", "434", "460", "534-over", "585"]

for s in ssps:
    concf = f"{input_dir}ssp{s}_conc_RCMIP.txt"
    emif = f"{input_dir}ssp{s}_em_RCMIP.txt"

    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(data_dir, "gases_v1RCMIP.txt"),
            "concentrations_file": concf,
            "emissions_file": emif,
            "nat_ch4_file": os.path.join(data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(data_dir, "natemis_n2o.txt"),
        },
    )
    cscm._run({"output_folder": outdir, "output_prefix": f"ssp{s}"})



import os

import numpy as np
import pandas as pd
import pytest

from ciceroscm import input_handler


def test_check_inputfiles():
    cfg = {"gaspamfile": "definitely_not_a_valid_path"}
    with pytest.raises(FileNotFoundError):
        cfg = input_handler.check_inputfiles(cfg)
    cfg = {"nat_ch4_file": "definitely_not_a_valid_path"}
    cfg = input_handler.check_inputfiles(cfg)
    assert cfg["nat_ch4_file"] == os.path.join(
        os.getcwd(), "input_OTHER", "NATEMIS", "natemis_ch4.txt"
    )
    cfg = {"concentration_file": "also_not_a_valid_path"}
    with pytest.raises(FileNotFoundError):
        cfg = input_handler.check_inputfiles(cfg)


def test_read_components(test_data_dir):
    df_gas = input_handler.read_components(
        os.path.join(test_data_dir, "gases_v1RCMIP.txt")
    )
    print(df_gas.columns)
    assert df_gas.columns.tolist() == [
        "EM_UNIT",
        "CONC_UNIT",
        "BETA",
        "ALPHA",
        "TAU1",
        "TAU2",
        "TAU3",
        "NAT_EM",
        "SARF_TO_ERF",
    ]
    assert len(df_gas.index) == 46


def test_read_forc(test_data_dir):
    forc = input_handler.read_forc(os.path.join(test_data_dir, "CO2_1pros.txt"))
    assert isinstance(forc, np.ndarray)
    assert len(forc) == 736

    forc = input_handler.read_forc(os.path.join(test_data_dir, "AR6_ERF_1750-2019.csv"))
    assert isinstance(forc, pd.DataFrame)
    total = forc["total"].to_numpy()
    assert len(total) == 270
    assert total[1] == 0.2866421077706115


def test_get_data_and_more(test_data_dir):
    ih = input_handler.InputHandler(
        {
            "forc_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
            "unicorn_emissions_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
        }
    )
    forc = ih.get_data("forc")
    assert isinstance(forc, np.ndarray)
    assert len(forc) == 736
    assert ih.optional_pam("forc")
    assert not ih.optional_pam("made_up_pam")
    assert not ih.conc_run()
    with pytest.raises(KeyError, match="No user or default data for emissions"):
        ih.get_data("emissions")
    with pytest.raises(KeyError, match="No user or default data for unicorn_emissions"):
        ih.get_data("unicorn_emissions")


def test_read_volc_sun(test_data_dir):
    ih = input_handler.InputHandler(
        {
            "forc_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
            "unicorn_emissions_file": os.path.join(test_data_dir, "CO2_1pros.txt"),
        }
    )

    # Testing volcano rf reading
    df_volc = ih.read_data_on_year_row(
        os.path.join(test_data_dir, "meanVOLCmnd_ipcc_NH.txt")
    )
    print(df_volc.columns.tolist)
    assert len(df_volc.index) == 351
    assert df_volc.columns.tolist() == list(range(12))

    # Testing solar rf reading:
    df_sun = ih.read_data_on_year_row(os.path.join(test_data_dir, "solar_IPCC.txt"))
    assert len(df_sun.index) == 351
    assert df_sun.columns.tolist() == [0]

    # Testing LUCalbedo rf reading
    df_luc = ih.read_data_on_year_row(os.path.join(test_data_dir, "IPCC_LUCalbedo.txt"))
    assert len(df_luc.index) == 351
    assert df_luc.columns.tolist() == [0]



from ciceroscm import concentrations_emissions_handler


def test_rs_and_rb_functions():
    assert concentrations_emissions_handler._rs_function(0) == 1.0
    assert concentrations_emissions_handler._rs_function(3.0) == 0.6884435390654896
    assert concentrations_emissions_handler._rb_function(0) == -3.599999999991197e-06



import os
import shutil
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "../", "src"))

from ciceroscm import CICEROSCM

data_dir = os.path.join(os.path.dirname(__file__), "../", "tests", "test-data")
outdir = os.path.join(os.getcwd(), "output_test") 
cscm = CICEROSCM(
    {
        "gaspam_file": os.path.join(data_dir, "gases_v1RCMIP.txt"),
        "nyend": 2100,
        "concentrations_file": os.path.join(data_dir, "ssp245_conc_RCMIP.txt"),
        "emissions_file": os.path.join(data_dir, "ssp245_em_RCMIP.txt"),
        "nat_ch4_file": os.path.join(data_dir, "natemis_ch4.txt"),
        "nat_n2o_file": os.path.join(data_dir, "natemis_n2o.txt"),
    },
)


cscm._run({"output_folder": outdir})




"""
Module for running CICEROSCM in parallel
"""

import logging
from concurrent.futures import ProcessPoolExecutor
from itertools import product

import pandas as pd
import scmdata
from openscm_runner.adapters.ciceroscm_py_adapter.make_scenario_data import (
    SCENARIODATAGETTER,
)
from openscm_runner.adapters.ciceroscm_py_adapter.read_results import CSCMREADER
from openscm_runner.adapters.utils._parallel_process import _parallel_process

from .._utils import check_numeric_pamset
from ..ciceroscm import CICEROSCM

LOGGER = logging.getLogger(__name__)


FRONT_SERIAL = 0
"""int: Number of serial runs to do before starting parallel runs"""

FRONT_PARALLEL = 0
"""int: Number of front parallel runs to do before starting full parallel runs"""


def _execute_run(cfgs, output_variables, scenariodata):
    """
    Run execution method to be used by parallel run

    Parameters
    ----------
    scenariodata : dict
        Dictionary of scenariodicts to run the cscm model for single run

    cfgs : dict
        Dict of configurations with which to run CICEROSCM for single run

    output_vars : list[str]
        Variables to output these follow ScmRun conventions

    Returns
    -------
    :obj:`ScmRun`
        :obj:`ScmRun` instance with all results.

    """
    scenariodata = check_numeric_pamset(
        {"nystart": 1750, "nyend": 2100, "emstart": 1850}, scenariodata
    )
    par_wrapper = CSCMParWrapper(scenariodata)
    try:
        out = par_wrapper.run_over_cfgs(cfgs, output_variables)
    finally:
        LOGGER.info("Finished run")
    return out


def run_ciceroscm_parallel(scendata, cfgs, output_vars, max_workers=4):
    """
    Run CICEROSCM in parallel

    Parameters
    ----------
    scendata : list[dict]
        List of scenariodicts to run the cscm model with
        Scenariodata with which to run

    cfgs : list[dict]
        List of configurations with which to run CICEROSCM
        Should be used for each scendata set

    output_vars : list[str]
        Variables to output, these follow ScmRun conventions

    max_workers : int
        Max number of parallel threads to run. Default value 4.

    Returns
    -------
    :obj:`ScmRun`
        :obj:`ScmRun` instance with all results.
    """
    LOGGER.info("Entered _parallel_ciceroscm")
    if len(scendata) >= max_workers or len(cfgs) * len(scendata) < max_workers:
        runs = [
            {"cfgs": cfgs, "output_variables": output_vars, "scenariodata": scen}
            for scen in scendata
        ]
    else:
        batch_size = len(cfgs) * len(scendata) // max_workers
        runs = [
            {"cfgs": cfgset, "output_variables": output_vars, "scenariodata": scen}
            for scen, cfgset in product(
                scendata,
                [cfgs[i : i + batch_size] for i in range(0, len(cfgs), batch_size)],
            )
        ]
    LOGGER.info("Running in parallel with up to %d workers", max_workers)

    with ProcessPoolExecutor(max_workers=max_workers) as pool:
        result = _parallel_process(
            func=_execute_run,
            configuration=runs,
            pool=pool,
            config_are_kwargs=True,
            # no front runs as these defeat the purpose with CICERO-SCM (because
            # it is only parallel on scenarios, not configs)
            front_serial=FRONT_SERIAL,
            front_parallel=FRONT_PARALLEL,
        )

    LOGGER.info("Appending CICERO-SCM results into a single ScmRun")
    result = scmdata.run_append([r for r in result if r is not None])

    return result


class CSCMParWrapper:  # pylint: disable=too-few-public-methods
    """
    CICEROSCM parallel run wrapper
    """

    def __init__(self, scenariodata):
        """
        Intialise CICEROSCM wrapper

        Parameters
        ----------
        scenariodata: dict
              Dictionary of scenariodata to initialise a ciceroscm with
              In addition to nystart and nyend, the scenariodata must contain
              "udir" a path to an utilities directory containing a gaspam file
              "scenname": A scenario name
        """
        self.udir = scenariodata["udir"]
        nystart = scenariodata["nystart"]
        nyend = scenariodata["nyend"]
        self.sdatagetter = SCENARIODATAGETTER(self.udir, nystart, nyend)
        self.resultsreader = CSCMREADER(nystart, nyend)
        self.cscm = CICEROSCM(scenariodata)
        self.scen = scenariodata["scenname"]
        self.model = scenariodata["scenname"]

    def run_over_cfgs(self, cfgs, output_variables):
        """
        Run over each configuration parameter set
        write parameterfiles, run, read results
        and make an ScmRun with results

        Parameters
        ----------
        cfgs : dict
            Dictionary of configurations with which to run CICEROSCM
            This should include dictionaries pamset_udm and a pamset_emiconc
            (though the latter may be empty for a forcing run). In addition
            it should include a string "Index" that identifies this
            configuration set

        output_variables : list[str]
            Variables to output, these follow ScmRun conventions

        Returns
        -------
             :obj:`ScmRun`
             :obj:`ScmRun` instance with all requested output variables
        """
        runs = []
        for pamset in cfgs:
            self.cscm._run(  # pylint: disable=protected-access
                {"results_as_dict": True},
                pamset_udm=pamset["pamset_udm"],
                pamset_emiconc=pamset["pamset_emiconc"],
            )
            for variable in output_variables:
                (
                    years,
                    timeseries,
                    unit,
                ) = self.resultsreader.get_variable_timeseries(
                    self.cscm.results, variable, self.sdatagetter
                )
                if isinstance(years, pd.DataFrame) and years.empty:  # pragma: no cover
                    continue  # pragma: no cover
                runs.append(
                    scmdata.ScmRun(
                        pd.Series(timeseries, index=years),
                        columns={
                            "climate_model": "CICERO-SCM-PY",
                            "model": self.model,
                            "run_id": pamset["Index"],
                            "scenario": self.scen,
                            "region": ["World"],
                            "variable": [variable],
                            "unit": [unit],
                        },
                    )
                )

        return scmdata.run_append(runs)



# Tests for different decay functions will go here...
import os
import shutil
import warnings

import numpy as np
import pandas as pd
import pandas.testing as pdt

from ciceroscm import CICEROSCM, pub_utils


def check_output_not_equal(
    output_dir,
    expected_output_dir,
    update_expected_files=False,
    rtol=1e-1,
    files=["output_em.txt", "output_conc.txt"],
):
    for filename in files:
        file_to_check = os.path.join(output_dir, filename)
        file_expected = os.path.join(expected_output_dir, filename)

        if update_expected_files:
            shutil.copyfile(file_to_check, file_expected)
        else:
            res = pd.read_csv(file_to_check, delim_whitespace=True)
            exp = pd.read_csv(file_expected, delim_whitespace=True)
            pdt.assert_index_equal(res.index, exp.index)

            try:
                pdt.assert_frame_equal(
                    res.T,
                    exp.T,
                    check_like=True,
                    rtol=rtol,
                )
            except AssertionError:
                pass
            else:
                raise AssertionError("DataFrames that should be different are equal")


def test_swap_out_rs_function(tmpdir, test_data_dir):
    with warnings.catch_warnings():
        cscm = CICEROSCM(
            {
                "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
                # "gaspam_file": os.path.join(test_data_dir, "gases_vupdate_2022_AR6.txt"),
                "nyend": 2100,
                "nystart": 1750,
                "emstart": 1850,
                "concentrations_file": os.path.join(
                    test_data_dir, "ssp245_conc_RCMIP.txt"
                ),
                "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
                "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
                "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
                "rs_function": lambda it, idtm: 0.5 + 0.5 * np.exp(-it / idtm / 100.0),
            },
        )
        warnings.simplefilter("error")
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:
    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 15.1,
            "akapa": 0.657,
            "cpi": 0.208,
            "W": 2.2,
            "beto": 6.9,
            "lambda": 0.606,
            "mixed": 107.0,
        },
        pamset_emiconc={
            "qbmb": 0.0,
            "qo3": 0.5,
            "qdirso2": -0.3701,
            "qindso2": -0.4163,
            "qbc": 0.163,
            "qoc": -0.084,
            "qh2o_ch4": 0.171,
        },
    )

    check_output_not_equal(
        outdir,
        os.path.join(test_data_dir, "ssp245_conc"),
        files=["output_conc.txt"],
    )


def test_swap_out_rb_function(tmpdir, test_data_dir):
    rb_decay = pub_utils.make_rb_function_from_arrays(
        [0.70211, 13.4141e-3, -0.71846, 2.9323e-3], [1 / 0.35, 20.0, 120 / 55.0, 100.0]
    )
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            # "gaspam_file": os.path.join(test_data_dir, "gases_vupdate_2022_AR6.txt"),
            "nyend": 2100,
            "nystart": 1750,
            "emstart": 1850,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "rb_function": rb_decay,
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:
    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 15.1,
            "akapa": 0.657,
            "cpi": 0.208,
            "W": 2.2,
            "beto": 6.9,
            "lambda": 0.606,
            "mixed": 107.0,
        },
        pamset_emiconc={
            "qbmb": 0.0,
            "qo3": 0.5,
            "qdirso2": -0.3701,
            "qindso2": -0.4163,
            "qbc": 0.163,
            "qoc": -0.084,
            "qh2o_ch4": 0.171,
        },
    )
    check_output_not_equal(
        outdir,
        os.path.join(test_data_dir, "ssp245_conc"),
        files=["output_conc.txt"],
    )


def test_swap_out_both_r_functions(tmpdir, test_data_dir):
    rb_decay = pub_utils.make_rb_function_from_arrays(
        [0.70211, 13.4141e-3, -0.71846, 2.9323e-3], [1 / 0.35, 20.0, 120 / 55.0, 100.0]
    )
    rs_decay_fair = pub_utils.make_rs_function_from_arrays(
        [0.24278, 0.13963, 0.089318, 0.03782, 0.035549],
        [1.2679, 5.2528, 18.601, 68.736, 232.3],
    )
    cscm = CICEROSCM(
        {
            "gaspam_file": os.path.join(test_data_dir, "gases_v1RCMIP.txt"),
            # "gaspam_file": os.path.join(test_data_dir, "gases_vupdate_2022_AR6.txt"),
            "nyend": 2100,
            "nystart": 1750,
            "emstart": 1850,
            "concentrations_file": os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"),
            "emissions_file": os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"),
            "nat_ch4_file": os.path.join(test_data_dir, "natemis_ch4.txt"),
            "nat_n2o_file": os.path.join(test_data_dir, "natemis_n2o.txt"),
            "rb_function": rb_decay,
            "rs_function": rs_decay_fair,
        },
    )
    # outdir_save = os.path.join(os.getcwd(), "output")
    outdir = str(tmpdir)
    # One year forcing:
    cscm._run(
        {"output_folder": outdir},
        pamset_udm={
            "rlamdo": 15.1,
            "akapa": 0.657,
            "cpi": 0.208,
            "W": 2.2,
            "beto": 6.9,
            "lambda": 0.606,
            "mixed": 107.0,
        },
        pamset_emiconc={
            "qbmb": 0.0,
            "qo3": 0.5,
            "qdirso2": -0.3701,
            "qindso2": -0.4163,
            "qbc": 0.163,
            "qoc": -0.084,
            "qh2o_ch4": 0.171,
        },
    )

    check_output_not_equal(
        outdir,
        os.path.join(test_data_dir, "ssp245_conc"),
        files=["output_conc.txt"],
    )



from ciceroscm import _utils


def test_check_numeric_pamset():
    required = {"Year": 2022, "number": 4.55, "test": 6.3e-4, "test2": 85}
    pamset = {
        "test": 7.8e-3,
        "test2": "string",
        "untouched": 44,
        "untouched_string": "Hello",
    }
    pamset2 = _utils.check_numeric_pamset(required, pamset)
    assert pamset2["Year"] == 2022
    assert pamset2["number"] == 4.55
    assert pamset2["test"] == 7.8e-3
    assert pamset2["test2"] == 85
    assert pamset2["untouched"] == 44
    assert pamset2["untouched_string"] == "Hello"


def test_cut_and_check_pamset():
    required = {"Year": 2022, "number": 4.55, "test": 6.3e-4, "test2": 85}
    pamset = {
        "test": 7.8e-3,
        "test2": "string",
        "untouched": 44,
        "untouched_string": "Hello",
    }
    used = {
        "test": 6,
        "test2": "string",
        "untouched": 44,
    }
    pamset2 = _utils.cut_and_check_pamset(
        required, pamset, used=used, cut_warnings=True
    )
    assert pamset2["Year"] == 2022
    assert pamset2["number"] == 4.55
    assert pamset2["test"] == 7.8e-3
    assert pamset2["test2"] == 85
    expected_keys = ["test", "test2", "untouched", "Year", "number"]
    for key in expected_keys:
        assert key in pamset2
    assert "untouched_string" not in pamset2



"""
Module to make parameter distributions
"""

import json
import logging

import numpy as np
from scipy.stats import qmc

LOGGER = logging.getLogger(__name__)

# TODO: sensible standard priors for q-terms
prior_flat = {
    "rlamdo": [5, 25],
    "akapa": [0.06, 0.08],
    "cpi": [0.161, 0.569],
    "W": [0.55, 2.55],
    "beto": [0, 7],
    "lambda": [2 / 3.71, 5 / 3.71],
    "mixed": [25, 125],
    "qbmb": [0, 2],
    "qo3": [0.4, 0.6],
    "qdirso2": [-0.55, -0.2],
    "qindso2": [-1.5, -0.5],
    "qbc": [0.1, 0.2],
    "qoc": [-0.1, -0.06],
    "qh2o_ch4": [0.08, 0.1],
    "aerosol_total": [-3.0, -0.5],
    "beta_f": [0.110, 0.465],
}
"""dict: Containing a default prior for parameters """

prior_flat_array = [
    [5, 25],
    [0.06, 0.08],
    [0.161, 0.569],
    [0.55, 2.55],
    [0, 7],
    [2 / 3.71, 5 / 3.71],
    [25, 125],
    [0, 2],
    [0.4, 0.6],
    [-0.55, -0.2],
    [-1.5, -0.5],
    [0.1, 0.2],
    [-0.1, -0.06],
    [0.08, 0.1],
]
"""array: same as prior_flat, but just as an array ordered according to standard ordering"""

ordering_standard = [
    "rlamdo",
    "akapa",
    "cpi",
    "W",
    "beto",
    "lambda",
    "mixed",
    "threstemp",
    "lm",
    "ldtime",
    "qo3",
    "qdirso2",
    "qindso2",
    "qbc",
    "qoc",
    "qbmb",
    "qh2o_ch4",
]
"""list: Containing a default ordering of parameters """

ordering_standard_forc = [
    "rlamdo",
    "akapa",
    "cpi",
    "W",
    "beto",
    "lambda",
    "mixed",
    "threstemp",
    "lm",
    "ldtime",
]
"""list: Containing a default ordering of parameters for forcing run"""
aerosols = ["qdirso2", "qindso2", "qbc", "qoc"]


class _ConfigDistro:
    """
    Class that holds distribution information for parameters and can be used
    to create sample parameter sets from
    """

    def __init__(
        self,
        distro_array=prior_flat_array,
        setvalues={
            "threstemp": 7.0,
            "lm": 40,
            "ldtime": 12,
            "qbmb": 0,
            "qh2o_ch4": 0.091915,
        },
        ordering="standard",
        options={
            "forc": False,
            "method": "latin",
            "aerosol-total": [-0.461590, -0.519163, 0.202893, -0.104478],
        },
    ):  # pylint:disable=dangerous-default-value
        """
        Intialise _ConfigDistro

        Parameters
        ----------
        distro_array: array
              2D array, with first index over the parameters according
              to the ordering given in the ordering parameter. Second index
              of length 2, and gives the ends of the interval for the
              parameter. If the method is latin, these are assumed to be
              extent of the parameter dimension. If the method is gaussian,
              these are assumed to be points one standard deviation away
              from the center of of the distribution for the parameter
              If the distro_array does not have correct type or dimensions
              the default prior will be used.
        setvalues: dict
              dictionary of parameters that will have a set value, keys
              are parameter names, values are their set values
        ordering: list
              list of parameter ordering. This  should match the ordering
              of values in the distro_array
        options: dict
              for optional key word arguments, such as forc, a bool which
              defines whether parameter distribution is meant for forcing
              runs. method, a string to define the configuration sampling
              method. If method='latin', latin hypercube is used, and this
              is the default method. If gaussian is chosen, samples are
              drawn from gaussian distributions for each of the
              parameters. This method will also be cosen if some other random
              string or object is sent for this keyword argument.
              And aerosol_total, which should be a four element array
              defining the proportions between the aerosol forcings for
              dirso2, indso2, bc and oc in that order
        """
        self.options = options
        if "forc" not in options:
            self.options["forc"] = False
        if "method" not in options:
            self.options["method"] = "latin"
        if "aerosol_total" in ordering and "aerosol_total" not in options:
            options["aerosol_total"] = [-0.461590, -0.519163, 0.202893, -0.104478]
        if "aerosol_total" in ordering:
            self.options["aerosol_total"] = np.array(options["aerosol_total"]) / np.sum(
                options["aerosol_total"]
            )
        if ordering == "standard":
            if self.options["forc"]:
                ordering = [
                    o for o in ordering_standard_forc if o not in set(setvalues)
                ]
            else:
                ordering = [o for o in ordering_standard if o not in set(setvalues)]
                if "aerosol_total" in ordering:
                    for aerosol in aerosols:
                        ordering.remove(aerosol)
        else:
            ordering = [o for o in ordering if o not in set(setvalues)]
        self.ordering = ordering
        self.prior = self._set_prior(distro_array)
        self.setvalues = setvalues
        self._set_pamsets_start()

    def _set_prior(self, distro_array):
        """
        Fill out the prior from a given distro_array

        Parameters
        ----------
        distro_array: array
              2D array, with first index over the parameters that according
              to the ordering given in the ordering parameter. Second index
              of length 2, and gives the ends of the interval for the
              parameter. If the method is latin, these are assumed to be
              extent of the parameter dimension. If the method is gaussian
              These are assumed to be points one standard deviation away
              from the center of of the distribution for the parameter
              If the distro_array does not have correct type or dimensions
              the default prior will be used.

        Returns
        -------
             array
             2D array of intervals for all the parameters that don't
             have setvalues according to the ordering.
        """
        prior = np.zeros([len(self.ordering), 2])
        len_given = 0
        try:
            len_given = len(distro_array[:, 0])
            prior[:len_given, :] = distro_array
        except (ValueError, TypeError):
            LOGGER.warning("distro_array not 2 dimensional, disregarding")
        for i, pam in enumerate(self.ordering[len_given:]):
            prior[len_given + i, :] = prior_flat[pam]
        return prior

    def _set_pamsets_start(self):
        """
        Set starting pamsets from the setvalues
        """
        self.pamset_udm_start = {}
        self.pamset_emiconc_start = {}
        for pam, value in self.setvalues.items():
            if pam in ordering_standard_forc:
                self.pamset_udm_start[pam] = value
            else:
                self.pamset_emiconc_start[pam] = value

    def get_samples_from_distro_latin(self, numvalues):
        """
        Get latin hypercube samples over the prior

        Parameters
        ----------
        numvalues: int
           number of values in returned sample

        Returns
        -------
        np.array
             latin hypercube of numvalues samples in the intervals
             given by the prior
        """
        sampler = qmc.LatinHypercube(d=len(self.ordering))
        samples_unit = sampler.random(n=numvalues)
        samples_scaled = np.array(
            qmc.scale(samples_unit, self.prior[:, 0], self.prior[:, 1])
        )
        return samples_scaled

    def get_samples_from_distro_gaussian(self, numvalues):
        """
        Get samples from gaussian distribution over the prior

        Parameters
        ----------
        numvalues: int
           number of values in returned sample

        Returns
        -------
        np.array
           numvalues samples drawn from gaussian distributions
           in the intervals for the parameters given by the prior
        """
        means = np.mean(self.prior, axis=1)
        std = self.prior[:, 1] - means
        samples = np.array(
            [
                np.random.normal(loc=means[i], scale=std[i], size=numvalues)
                for i in range(len(self.prior))
            ]
        ).transpose()
        return samples

    def make_config_list(self, numvalues, json_fname="No", indexer_pre=""):
        """
        Make configuration list from samples over the distribution

        Parameters
        ----------
        numvalues: int
           number of values in returned configuration set
        json_fname: str
           path to file to output configurations
        indexer_pre: str
           prefix to put on the Index for the samples

        Returns
        -------
        list
           list of numvalue configuration dictionaries with parameter
           values drawn according to the distribution objects prior and
           distribution methods, along with set values for the parameters
           that are defined with setvalues
        """
        config_list = [None] * numvalues
        if self.options["method"] == "latin":
            samples = self.get_samples_from_distro_latin(numvalues)
        else:
            samples = self.get_samples_from_distro_gaussian(numvalues)

        for i in range(numvalues):
            pamset_udm = self.pamset_udm_start
            pamset_emiconc = self.pamset_emiconc_start
            for j, pam in enumerate(self.ordering):
                if pam in ordering_standard_forc:
                    pamset_udm[pam] = samples[i, j]
                elif not self.options["forc"]:
                    if pam == "aerosol_total":
                        for anum, aerosol in enumerate(aerosols):
                            pamset_emiconc[aerosol] = (
                                samples[i, j] * self.options["aerosol_total"][anum]
                            )
                    else:
                        pamset_emiconc[pam] = samples[i, j]

            config_list[i] = {
                "pamset_udm": pamset_udm.copy(),
                "pamset_emiconc": pamset_emiconc.copy(),
                "Index": f"{indexer_pre}{i}",
            }
        if json_fname != "No":
            with open(json_fname, "w", encoding="utf-8") as wfile:
                json.dump(config_list, wfile)
        return config_list



import os

import numpy as np
import pytest

from ciceroscm import input_handler, pub_utils


def test_making_biotic_decay_function():
    rb_C = np.array([0.70211, 13.4141e-3, -0.71846, 2.9323e-3])
    rb_T = np.array([1 / 0.35, 20.0, 120 / 55.0, 100.0])
    rb_func = pub_utils.make_rb_function_from_arrays(rb_C, rb_T)
    expected = (
        0.70211 * np.exp(-0.35)
        + 13.4141e-3 * np.exp(-1 / 20.0)
        - 0.71846 * np.exp(-55.0 / 120)
        + 2.9323e-3 * np.exp(-1 / 100.0)
    )
    print(rb_func(24, 24))
    print(expected)
    assert rb_func(0, 24) == np.sum(rb_C)
    assert rb_func(24, 24) == np.dot(rb_C, np.exp(-1.0 / rb_T))
    assert (rb_func(24, 24) - expected) / expected < 1.0e-9


def test_making_carbon_pool_decay_function():
    rs_C = np.array([0.24278, 0.13963, 0.089318, 0.03782, 0.035549])
    rs_T = np.array([1.2679, 5.2528, 18.601, 68.736, 232.3])
    rs_func = pub_utils.make_rs_function_from_arrays(rs_C, rs_T)
    expected = (
        (1 - np.sum(rs_C))
        + 0.24278 * np.exp(-1 / 1.2679)
        + 0.13963 * np.exp(-1 / 5.2528)
        + 0.089318 * np.exp(-1 / 18.601)
        + 0.03782 * np.exp(-1 / 68.736)
        + 0.035549 * np.exp(-1 / 232.3)
    )
    assert rs_func(0, 24) == 1.0
    assert rs_func(1000, 24) < 1.0
    print(expected)
    print(rs_func(24, 24))
    assert rs_func(24, 24) == (1 - np.sum(rs_C)) + np.dot(rs_C, np.exp(-1.0 / rs_T))
    assert (rs_func(24, 24) - expected) / expected < 1e-7


def test_error_management_decay_functions():
    rs_T = "Not an array"
    rs_C = [1, 4, -5]
    with pytest.raises(TypeError):
        pub_utils.make_rs_function_from_arrays(rs_C, rs_T)
    rs_T = [3, 4]
    with pytest.raises(
        ValueError,
        match="Coefficient array length must be equal to exponent array length",
    ):
        rs = pub_utils.make_rs_function_from_arrays(rs_C, rs_T)
    with pytest.raises(ValueError, match="All timescales must be positive numbers"):
        rb = pub_utils.make_rb_function_from_arrays(rs_C, rs_C)
    with pytest.raises(ValueError, match="All timescales must be positive numbers"):
        rs = pub_utils.make_rs_function_from_arrays(rs_C, rs_C)
    rs_T = [1, 20, 100]
    with pytest.raises(ValueError, match="All coefficients must be positive"):
        rs = pub_utils.make_rs_function_from_arrays(rs_C, rs_T)
    rb = pub_utils.make_rb_function_from_arrays(rs_C, rs_T)
    assert rb(0, 100) == np.sum(rs_C)
    with pytest.raises(
        ValueError,
        match="Sum of carbon decay coefficients must be less than or equal to 1",
    ):
        rs = pub_utils.make_rs_function_from_arrays(rs_T, rs_T)
    rs_C = [0.25, 0.25, 0.25]
    rs = pub_utils.make_rs_function_from_arrays(rs_C, rs_T)
    assert rs(0, 5364) == 1.0


def test_find_num_cl_in_hcfc():
    assert pub_utils.find_num_cl_in_hcfc("CFC-11") == 3
    assert pub_utils.find_num_cl_in_hcfc("CFC-12") == 2
    assert pub_utils.find_num_cl_in_hcfc("CFC-113") == 3
    assert pub_utils.find_num_cl_in_hcfc("CFC-114") == 2
    assert pub_utils.find_num_cl_in_hcfc("CFC-115") == 1
    assert pub_utils.find_num_cl_in_hcfc("HCFC-22") == 1
    assert pub_utils.find_num_cl_in_hcfc("HCFC-141b") == 2
    assert pub_utils.find_num_cl_in_hcfc("HCFC-123") == 2
    assert pub_utils.find_num_cl_in_hcfc("HCFC-142b") == 1


def test_make_cl_and_br_dictionaries(test_data_dir):
    df_gas = input_handler.read_components(
        os.path.join(test_data_dir, "gases_v1RCMIP.txt")
    )
    chlor_dict, brom_dict = pub_utils.make_cl_and_br_dictionaries(df_gas.index)
    assert brom_dict == {"H-1211": 1, "H-1301": 1, "CH3Br": 1, "H-2402": 2}
    chlor_dict_old = {
        "CFC-11": 3,
        "CFC-12": 2,
        "CFC-113": 3,
        "CFC-114": 2,
        "CFC-115": 1,
        "CCl4": 4,
        "CH3CCl3": 3,
        "HCFC-22": 1,
        "HCFC-141b": 2,
        "HCFC-123": 2,
        "HCFC-142b": 1,
    }
    assert chlor_dict != chlor_dict_old
    chlor_dict_old["H-1211"] = 1
    assert chlor_dict == chlor_dict_old



#Plot concentration and compare to CMIP6 input 
#Meinshausen, M., Vogel, E., Nauels, A., Lorbacher, K., Meinshausen,
#N., Etheridge, D. M., Fraser, P. J., Montzka, S. A., Rayner,
#P. J., Trudinger, C. M., Krummel, P. B., Beyerle, U., Canadell,
#J. G., Daniel, J. S., Enting, I. G., Law, R. M., Lunder, C. R.,
#O'Doherty, S., Prinn, R. G., Reimann, S., Rubino, M., Velders,
#G. J. M., Vollmer, M. K., Wang, R. H. J., and Weiss,
#R.: Historical greenhouse gas concentrations for climate
#modelling (CMIP6), Geosci. Model Dev., 10,2057-2116,
#10.5194/gmd-10-2057-2017, 2017.

#Several components here not included in the CSCM
#Only compared the components with identical names.

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

plt.rcParams['font.size'] = 4

#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_conc=pd.read_csv(outdir+'/output_conc.txt', sep='\t', index_col=0)
#print(df_conc)

#Read components, to get the units:
inputdir = '/div/no-backup/users/ragnhibs/ciceroscm/tests/test-data/'
df_comp =pd.read_csv(inputdir + 'gases_v1RCMIP.txt', sep='\t', index_col=0)
#print(df_comp.loc['CO2']['EM_UNIT'])

print(df_comp.index)

#Selected components are those with identical component names as in CMIP6 file.
complist=['CFC-11', 'CFC-12', 'CFC-113', 'CFC-114',
          'CFC-115', 'CH3Br', 'CCl4', 'CH3CCl3', 'HCFC-22', 'HCFC-141b',
          'HCFC-142b', 
          'C2F6', 'C6F14', 'CF4', 'SF6']

compskiped = ['HCFC-123',          
              'H-1211', 'H-1301', 'H-2402','HFC125','HFC134a',
              'HFC143a','HFC227ea', 'HFC23', 'HFC245fa','HFC32',
              'HFC4310mee']

cmip6_conc = pd.read_excel('data_compare/Supplementary_Table_UoM_GHGConcentrations-1-1-0_annualmeans_v23March2017.xls',
                           sheet_name='historical-annualmean-Global',
                           header=21,index_col=0)
cmip6_conc.index.name='Years'
print(cmip6_conc)


fig, axs = plt.subplots(nrows=4, ncols=4,sharex=True,figsize=(12,16))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, concentrations')
for i,comp in enumerate(complist):
    df_conc[comp].plot(ylabel= comp +' ['+df_comp.loc[comp]['CONC_UNIT']+']',
                       ax=axs[i],label=scen)
    
    cmip6_conc[comp].plot(ax=axs[i],color='black',label='CMIP6input')
    axs[i].set_title(comp)
    axs[i].legend()
    axs[i].set_ylim(bottom=0)

df_conc[compskiped].plot(ylabel='ppt',ax=axs[15])    
axs[15].set_title('Rest of CSCM components')
axs[15].legend(loc='lower left')
plt.show()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
sys.path.append(os.path.join(os.path.dirname(__file__), "../../src/ciceroscm"))
from _version import get_versions  # isort:skip # append path before


# -- Project information -----------------------------------------------------

project = "Ciceroscm"
authors = ", ".join(["Ane Nordlie Johansen", "Benjaminn M. Sanderson", "Marit Sandstad", "Ragnhild Bieltvedt Skeie"])
copyright_year = "2022"
copyright = "{}, {}".format(copyright_year, authors)
author = authors

# The short X.Y version
version = get_versions()["version"].split("+")[0]
# The full version, including alpha/beta/rc tags
release = get_versions()["version"]


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.intersphinx",
    "sphinx.ext.napoleon",  # pass numpy style docstrings
    "sphinx_click.ext",
]
autodoc_default_options = {
    "inherited-members": None,
    "members": None,
    "private-members": None,
    "show-inheritance": None,
    "undoc-members": None,
}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["build"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}

html_context = {
    "display_github": False,
    "github_user": "ciceroOslo",
    "github_repo": "ciceroscm",
    "github_version": "main",
    "conf_py_path": "/docs/source",
}

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "ciceroscmdoc"


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, "ciceroscm.tex", "CICEROSCM Documentation", author, "manual")
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, "ciceroscm", "CICEROSCM Documentation", [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "ciceroscm",
        "CiceroSCM  Documentation",
        author,
        "ciceroscm",
        "Python port version of the CiceroSCM simple climate model",
        "Miscellaneous",
    )
]

# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]


# -- Extension configuration -------------------------------------------------

coverage_write_headline = False  # do not write headlines.

# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "numpy": ("https://docs.scipy.org/doc/numpy", None),
    "pandas": ("https://pandas.pydata.org/pandas-docs/stable", None),
    "python": ("https://docs.python.org/3", None),
    #"pyam": ("https://pyam-iamc.readthedocs.io/en/latest", None), #Put back if in use later
    #"scmdata": ("https://scmdata.readthedocs.io/en/latest", None), #Put back if in use later
    # "pint": ("https://pint.readthedocs.io/en/latest", None), # no full API doc here, unfortunately
}
napoleon_google_docstring = False
napoleon_numpy_docstring = True
set_type_checking_flag = False

# -- Options for todo extension ----------------------------------------------

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# # -- Logos -------------------------------------------------------------------

# html_logo = "_static/logo_200px_wide.png"
# latex_logo = "_static/logo.png"


# -- Misc configuration -------------------------------------------------

rst_epilog = """
.. |CO2| replace:: CO\ :sub:`2`\
"""



import os

import pandas as pd

from ciceroscm import input_handler
from ciceroscm.parallel._configdistro import _ConfigDistro
from ciceroscm.parallel.calibrator import Calibrator


def test_calibrator(test_data_dir):
    gaspam_data = input_handler.read_components(
        os.path.join(test_data_dir, "gases_v1RCMIP.txt")
    )
    conc_data = input_handler.read_inputfile(
        os.path.join(test_data_dir, "ssp245_conc_RCMIP.txt"), True, 1750, 2100
    )
    ih = input_handler.InputHandler({"nyend": 2050, "nystart": 1900, "emstart": 2015})
    em_data = ih.read_emissions(os.path.join(test_data_dir, "ssp245_em_RCMIP.txt"))
    nat_ch4_data = input_handler.read_natural_emissions(
        os.path.join(test_data_dir, "natemis_ch4.txt"), "CH4"
    )
    nat_n2o_data = input_handler.read_natural_emissions(
        os.path.join(test_data_dir, "natemis_n2o.txt"), "N2O"
    )
    scendata = {
        "gaspam_data": gaspam_data,
        "nyend": 2030,
        "nystart": 1900,
        "emstart": 2015,
        "concentrations_data": conc_data,
        "nat_ch4_data": nat_ch4_data,
        "nat_n2o_data": nat_n2o_data,
        "emissions_data": em_data,
        "udir": test_data_dir,
        "scenname": "ssp245",
    }
    calibdata = pd.DataFrame(
        data={
            "Variable Name": [
                "Heat Content|Ocean",
                "Surface Air Ocean Blended Temperature Change",
            ],
            "Yearstart_norm": [1971, 1961],
            "Yearend_norm": [1971, 1990],
            "Yearstart_change": [2018, 2000],
            "Yearend_change": [2018, 2019],
            "Central Value": [320.69251537323, 0.5372],
            "sigma": [17.020342912051203, 0.039028311931729676],
        }
    )
    # calibdata = pd.DataFrame(data = {'Variable Name': ["Heat Content|Ocean"], 'Yearstart_norm': [1971], "Yearend_norm":[1971], "Yearstart_change":[2018], "Yearend_change":[2018], "Central Value": [320.69251537323], "sigma": [17.020342912051203]})
    testconfig = _ConfigDistro(
        distro_array=[],
        ordering=["aerosol_total", "W", "lambda", "beta_f"],
        setvalues={
            "threstemp": 7.0,
            "lm": 40,
            "ldtime": 12,
            "qbmb": 0,
            "qo3": 0.5,
            "qh2o_ch4": 0.091915,
            "rlamdo": 16,
            "akapa": 0.634,
            "cpi": 0.4,
            "beto": 3.5,
            "mixed": 60,
        },
        options={"aerosol_total": [-0.36, -0.97, 0.16, -0.08]},
    )
    assert testconfig.ordering == ["aerosol_total", "W", "lambda", "beta_f"]
    calibrator = Calibrator(calibdata, testconfig, scendata)
    drawn_cfgs = calibrator.get_n_samples(
        1, current_samples=[], kept_configs=[], recurse_num=0
    )
    assert len(drawn_cfgs) >= 1
    testconfig = _ConfigDistro(
        distro_array=[],
        ordering=["W", "lambda"],
        setvalues={
            "threstemp": 7.0,
            "lm": 40,
            "ldtime": 12,
            "qbmb": 0,
            "qo3": 0.5,
            "qh2o_ch4": 0.091915,
            "rlamdo": 16,
            "akapa": 0.634,
            "cpi": 0.4,
            "beto": 3.5,
            "mixed": 60,
        },
    )
    assert testconfig.ordering == ["W", "lambda"]
    calibrator = Calibrator(calibdata, testconfig, scendata)
    drawn_cfgs = calibrator.get_n_samples(
        3, current_samples=[], kept_configs=[], recurse_num=0
    )
    assert len(drawn_cfgs) >= 3



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np


#Read results:
outdir = '/div/no-backup/users/ragnhibs/ciceroscm/scripts/output_test/'
scen = 'test'
df_ohc=pd.read_csv(outdir+'/output_ohc.txt', sep='\t', index_col=0)
print(df_ohc)


fig, axs = plt.subplots(nrows=1, ncols=2,sharex=True,figsize=(12,6))
axs=axs.flatten()
fig.suptitle('CICERO SCM simulation, Ocean heat content')


comp = 'OHC700'
df_ohc[comp].plot(ylabel='OHC [$10^{22}$ J]',ax=axs[0],label=scen)
axs[0].set_title(comp)


comp = 'OHCTOT'
df_ohc[comp].plot(ylabel='OHC [$10^{22}$ J]',ax=axs[1],label=scen)
axs[1].set_title(comp)

axs[0].legend()
axs[1].legend()

plt.show()



exit()
